<?php
namespace Phox_Host\Builder;

use function Phox_Host\phox_host_core;

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}

/**
 * Theme Builder Conditions
 *
 * Register theme builder conditions
 *
 * @package Builder
 * @since 1.7.8
 */
class Conditions_Manager{

    /**
     * conditions_list
     *
     * @since 1.7.8
     * @var array
     */
    private $conditions_list = [];

    /**
     * conditions option name
     *
     * @since 1.7.8
     * @var string
     */
    public $option_key = 'wdes_builder_conditions';

    public function __construct()
    {
        $this->register_conditions();
        add_action( 'elementor/editor/after_save', [$this, 'update_post_conditions']);
        add_action( 'wp_trash_post',               [$this, 'remove_post_from_conditions']);
        add_action( 'transition_post_status',      [$this, 'remove_post_from_conditions_on_change_status'], 10, 3 );
    }

    /**
     * Register Condition
     *
     * @since 1.7.8
     * @return void
     */
    public function register_conditions()
    {

        $base_path = Phox_HOST_PATH . 'includes/builder/conditions/';

        require $base_path . 'base_condition.php';

        $conditions = [
            '\Phox_Host\Builder\Conditions\Entire_Site'                            => $base_path . 'entire_site.php',
            '\Phox_Host\Builder\Conditions\Singular_Page'                          => $base_path . 'singular_page.php',
            '\Phox_Host\Builder\Conditions\Singular_Post'                          => $base_path . 'singular_post.php',
            '\Phox_Host\Builder\Conditions\Singular_Front_Page'                    => $base_path . 'singular_front_page.php',
            '\Phox_Host\Builder\Conditions\Singular_404'                           => $base_path . 'singular_404.php',
            '\Phox_Host\Builder\Conditions\Archive_All'                            => $base_path . 'archive_all.php',
            '\Phox_Host\Builder\Conditions\Archive_Search'                         => $base_path . 'archive_search.php',
        ];

        foreach ( $conditions as $class => $files ){

            require $files ;

            $this->register_condition($class);

        }


    }

    /**
     * Register Conditions
     *
     * @since 1.7.8
     * @return string
     */
    public function register_condition(string $class)
    {
        $instance = new $class;
        $this->conditions_list[ $instance->id() ] = $instance;
    }

    /**
     * Get Conditions
     *
     * @since 1.7.8
     * @return string
     */
    public function get_condition( $condition_id )
    {
        return $this->conditions_list[$condition_id] ?? false;
    }

    /**
     * Get General Options
     *
     * @since 1.7.8
     * @return array
     */
    public function get_general_options ()
    {
        return [
            'entire'   => __( 'Entire', 'phox-host' ),
            'singular' => __( 'Singular', 'phox-host' ),
            'archive'  => __( 'Archive', 'phox-host' ),
        ];
    }

    /**
     * Register Controls
     *
     * @since 1.7.8
     * @return void
     */
    public function register_controls ($controls_manager)
    {
        if ( ! $controls_manager ) {
            return;
        }

        $groups = $this->get_general_options();

        $prepared_data =$this->prepare_conditions_for_controls();
        $default       = ['' => esc_html__( 'Select...', 'phox-host' )];
        $general       = $default + $groups;

        $controls_manager->add_control(
            'conditions_top',
            [
                'label'   => esc_html__( 'General', 'phox-host' ),
                'type'    => \Elementor\Controls_Manager::SELECT,
                'default' => '',
                'options' => $general,
            ]
        );

        foreach ( $prepared_data as $group => $options ){

            if( empty($options) ){
                continue;
            }

            if( ! isset($groups[$group]) ){
                continue;
            }

            $condition =[
                'conditions_top' => $group
            ];

            $control_name = 'condition_sub_' . $group;


            $controls_manager->add_control(
                $control_name,
                array(
                    'label'     => $general[ $group ],
                    'type'      => \Elementor\Controls_Manager::SELECT,
                    'default'   => '',
                    'options'   =>  array_map( function ($option) { return $option['label']; } ,$options),
                    'condition' => $condition,
                )
            );

            foreach ( $options as $id => $data ){

                $init = $this->conditions_list[ $id ];
                $controls = $init->controls();

                if ( ! empty( $controls ) ) {

                    foreach ( $controls as $control_id => $control_options ) {
                        $_id         = 'conditions_' . $id . '_' . $control_id;
                        $child_cond = ! empty( $control_options['condition'] ) ? $control_options['condition'] : array();

                        $control_options['condition'] = array_merge(
                            $child_cond,
                            [$control_name => $id],
                            $condition
                        );

                        $controls_manager->add_control( $_id, $control_options );
                    }

                }

            }

        }

    }

    public function prepare_conditions_for_controls()
    {

        $stored_conditions = [];

        foreach ( $this->conditions_list as $id => $init ){

            $group = $init->group();


            if( ! isset( $stored_conditions[$group] ) ){

                $stored_conditions[$group] = [];

            }


            $current =[
              'label' => $init->label()
            ];

            $stored_conditions[$group][$id] =$current;


        }

        return $stored_conditions;

    }

    /**
     * Update Post Conditions
     *
     * It runs after post is saved
     *
     * @since 1.7.8
     * @return void
     */
    public function update_post_conditions($post_id){

        $post = get_post( $post_id );

        if ( phox_host_core()->theme_builder->slug() !== $post->post_type && phox_host_core()->theme_builder_popup->slug() !== $post->post_type ) {
            return;
        }

        if ( 'publish' !== $post->post_status ) {
            return;
        }

        $type      = get_post_meta( $post_id, '_elementor_template_type', true );
        $sanitized = $this->post_conditions($post_id);
        $saved     = get_option( $this->option_key, []);
        $saved     = $this->remove_post_from_conditions_array( $post_id, $saved );
        if ( ! isset( $saved[ $type ] ) ) {
            $saved[ $type ] = array();
        }

        $saved[ $type ][ $post_id ] = $sanitized;

        update_option( $this->option_key, $saved, true );

    }

    /**
     * Post sanitize
     *
     *
     * @param $post_id
     * @return array
     * @since 1.7.8
     */
    public function post_conditions($post_id): array
    {
        $group      = '';
        $conditions = get_post_meta( $post_id, '_elementor_page_settings', true );

        $sanitized  = array();

        if ( ! $conditions ) {
            $conditions = array();
        }

        if ( isset( $conditions['conditions_top'] ) ) {
            $group             = $conditions['conditions_top'];
            $sanitized['main'] = $group;

            unset( $conditions['conditions_top'] );
        }

        foreach ( $conditions as $condition => $value ) {
            if ( false === strpos( $condition, 'condition_' ) ) {
                continue;
            }
            if ( 'condition_sub_' . $group === $condition ) {

                $args   = array();
                $prefix = 'conditions_' . $value . '_';

                foreach ( $conditions as $condition_item => $value_item ) {

                    if ( false === strpos( $condition_item, $prefix ) ) {
                        continue;
                    }

                    $args[ str_replace( $prefix, '', $condition_item ) ] = $value_item;
                }

                $sanitized[ $value ] = $args;

            }

        }
        return $sanitized;

    }

    /**
     * Remove Post From Conditions Array
     *
     * @param $post_id
     * @param $conditions
     * @since 1.7.8
     * @return array|mixed
     */
    public function remove_post_from_conditions_array( $post_id = 0, $conditions = array() ) {

        foreach ( $conditions as $type => $type_conditions ) {
            if ( array_key_exists( $post_id, $type_conditions ) ) {
                unset( $conditions[ $type ][ $post_id ] );
            }
        }

        return $conditions;

    }

    /**
     * Remove Post From Site Conditions
     *
     * @param $post_id
     * @return void
     */
    public function remove_post_from_conditions( $post_id = 0 ) {

        $conditions = get_option( $this->option_key, array() );
        $conditions = $this->remove_post_from_conditions_array( $post_id, $conditions );

        update_option( $this->option_key, $conditions, true );
    }

    /**
     * Remove when change status
     *
     * @param $new_status
     * @param $old_status
     * @param $post
     * @return void
     */
    public function remove_post_from_conditions_on_change_status( $new_status, $old_status, $post ) {

        if ( phox_host_core()->theme_builder->slug() !== $post->post_type ) {
            return;
        }

        if ( in_array( $new_status, array( 'publish', 'trash' ) ) ) {
            return;
        }

        $this->remove_post_from_conditions( $post->ID );
    }

    /**
     * Get Conditions
     *
     * @param $type string select one of these types [wdes_header, wdes_footer, wdes_popup]
     * @return array
     */
    public function get_site_conditions($type){

        $conditions = [];

        $site_conditions = get_option( $this->option_key, [] );

        if ( empty( $site_conditions ) && empty( $site_conditions[$type] ) ) {
            return [];
        }

        $site_popup_conditions = $site_conditions[$type] ?? false;

        if($site_popup_conditions){

            $raw_conditions = array_map( function( $popup_data ) {

                if ( ! isset( $popup_data['main'] ) ) {
                    return $popup_data = [
                        'conditions'    => $popup_data,
                    ];
                }

                return $popup_data;
            }, $site_popup_conditions );

            foreach ($raw_conditions as $post_id => $condition){

                if( ! isset($condition['conditions']) ){

                    $index = 0;
                    $keys = array_keys($condition);

                    $conditions[$post_id]['conditions'][$index]['group'] = $condition[$keys[0]] ;
                    $conditions[$post_id]['conditions'][$index]['subGroup'] = $keys[1] ;
                    $conditions[$post_id]['conditions'][$index]['subGroupValue'] = $condition[$keys[1]] ;

                }else {

                    $conditions[$post_id] = $condition;

                }

            }

        }


        return $conditions;

    }

    public function  find_matched_by_conditions($type){

        $conditions = $this->get_site_conditions($type);

        $posts_id_list = [];

        foreach ( $conditions as $id => $data ) {

            $item_conditions = $data['conditions'];

            if ( empty( $item_conditions ) ) {
                continue;
            }


            $item_conditions = array_map( function( $condition)  {

                $sub_group = $condition['subGroup'];

                $instance = $this->get_condition( $sub_group );

                if ( ! $instance ) {
                    $condition['match'] = true;

                    return $condition;
                }

                $sub_group_value = isset( $condition['subGroupValue'] ) ? $condition['subGroupValue'] : '';

                $instance_check = call_user_func( array( $instance, 'check' ), $sub_group_value, $sub_group );

                $condition['match'] = $instance_check;

                return $condition;

            }, $item_conditions );


            $includes_matches = [];

            foreach ( $item_conditions as $key => $condition ) {
                $includes_matches[] = $condition['match'];
            }

            $is_included = ! empty( $includes_matches ) &&  in_array( true, $includes_matches );

            // final check
            if ( $is_included  ) {
                $posts_id_list [] = apply_filters( 'wdes-phox-host-builder/integration/multilingual/template_id', $id );
            }

        }

        if ( ! empty( $posts_id_list ) ) {
            return $posts_id_list;
        }

        return false;

    }

    /**
     * Get active conditions for passed post
     *
     * @param  [type] $post_id [description]
     * @return [type]          [description]
     */
    public function post_conditions_verbose( $post_id = null ) {

        $conditions = $this->post_conditions( $post_id );

        if ( empty( $conditions['main'] ) ) {
            return;
        }

        if ( 'entire' === $conditions['main'] ) {
            return __( 'Entire Site', 'phox-host' );
        }

        unset( $conditions['main'] );

        $condition_keys = array_keys( $conditions );
        $verbose        = '';

        foreach ( $condition_keys as $key ) {

            $instance     = $this->get_condition( $key );
            $verbose_args = $instance->get_verbose_items( $conditions[ $key ] );

            if ( ! empty( $verbose_args ) ) {
                $verbose_args = ': ' . $verbose_args;
            }

            $verbose .= sprintf( '<div>%1$s%2$s</div>', $instance->label(), $verbose_args );

        }

        return $verbose;

    }



}
