<?php
namespace Phox_Host\Builder\Popup\Render\Modules;

use Phox_Host\Builder\Popup\Render\Base_Render;

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}

Class Front_Content extends Base_Render{

    /**
     * @var string
     */
    protected $name = 'elementor-template-render';

    /**
     * @var array
     */
    public $depended_styles = [];

    /**
     * @var array
     */
    public $depended_scripts = [];

    /**
     * @var array
     */
    public $elementor_widgets = [];

    /**
     * @var bool
     */
    public $post_css_exist = false;

    /**
     * Get Name
     *
     * @return mixed|string
     */
    public function get_name() {
        return $this->name;
    }

    /**
     * [render description]
     * @return [type] [description]
     */
    public function render() {

        if ( ! defined( 'ELEMENTOR_VERSION' ) ) {
            echo __( 'Elementor not installed', 'phox-host' );

            return false;
        }

        if ( $this->is_editor() ) {
            echo __( 'Elementor editor content is not available in the Blocks Editor', 'phox-host' );

            return false;
        }

        $popup_id   = $this->get( 'popup_id', false );
        $with_css   = $this->get( 'with_css', false );
        $is_content = $this->get( 'is_content', true );

        if ( ! $popup_id || ! $is_content ) {
            return false;
        }

        $before_styles_queue  = wp_styles()->queue;
        $before_scripts_queue = wp_scripts()->queue;

        $content              = \Elementor\Plugin::instance()->frontend->get_builder_content_for_display( $popup_id, $with_css );

        $after_styles_queue   = wp_styles()->queue;
        $after_scripts_queue  = wp_scripts()->queue;

        $styles_depends         = array_diff( $after_styles_queue, $before_styles_queue );
        $script_depends         = array_diff( $after_scripts_queue, $before_scripts_queue );

        $this->depended_styles  = array_merge( $this->depended_styles, $styles_depends );
        $this->depended_scripts = array_merge( $this->depended_scripts, $script_depends );

        echo do_shortcode( $content );

    }

    /**
     * Find widgets script handlers
     *
     * @param $elements_data
     * @return void
     */
    public function find_widgets_script_handlers( $elements_data ) {

        foreach ( $elements_data as $element_data ) {

            if ( 'widget' === $element_data['elType'] ) {
                $widget                    = \Elementor\Plugin::$instance->elements_manager->create_element_instance( $element_data );
                $this->elementor_widgets[] = $widget->get_name();
            } else {
                $element = \Elementor\Plugin::$instance->elements_manager->create_element_instance( $element_data );
                $childrens = $element->get_children();

                foreach ( $childrens as $key => $children ) {
                    $children_data[ $key ] = $children->get_raw_data();
                    $this->find_widgets_script_handlers( $children_data );
                }
            }
        }
    }

    /**
     * Get registered style obj by handler
     *
     * @param $handler
     * @return false|mixed
     */
    public function get_registered_style_obj_by_handler( $handler ) {

        if ( isset( wp_styles()->registered[ $handler ] ) ) {
            $src = wp_styles()->registered[ $handler ]->src;

            if ( ! preg_match( '|^(https?:)?//|', $src ) && ! ( wp_styles()->content_url && 0 === strpos( $src, wp_styles()->content_url ) ) ) {
                wp_styles()->registered[ $handler ]->src = wp_styles()->base_url . $src;
            }

            return wp_styles()->registered[ $handler ];
        }

        return false;
    }

    /**
     * Get registered script object by handler
     *
     * @param $handler
     * @return false|mixed
     */
    public function get_registered_script_obj_by_handler( $handler ) {

        if ( isset( wp_scripts()->registered[ $handler ] ) ) {
            $src = wp_scripts()->registered[ $handler ]->src;

            if ( ! preg_match( '|^(https?:)?//|', $src ) && ! ( wp_scripts()->content_url && 0 === strpos( $src, wp_scripts()->content_url ) ) ) {
                wp_scripts()->registered[ $handler ]->src = wp_scripts()->base_url . $src;
            }

            return wp_scripts()->registered[ $handler ];
        }

        return false;
    }

    /**
     * Get content script data
     *
     * @param $popup_id
     * @return array|array[]
     */
    public function get_content_script_data( $popup_id = false ) {

        if ( ! $popup_id ) {
            return [
                'depends' => [],
                'widgets' => [],
            ];
        }

        $is_deps = $this->get( 'is_script_deps', true );

        if ( ! $is_deps ) {
            return [
                'depends' => [],
                'widgets' => [],
            ];
        }

        $data_cache = get_post_meta( $popup_id, '_is_script_deps', true );

        if ( ! empty( $data_cache ) ) {
            return $data_cache;
        }

        $document = \Elementor\Plugin::$instance->documents->get( $popup_id );

        if ( ! $document ) {
            return [
                'depends' => [],
                'widgets' => [],
            ];
        }

        $elements_data = $document->get_elements_raw_data();

        if ( empty( $elements_data ) ) {
            return [
                'depends' => [],
                'widgets' => [],
            ];
        }

        $this->find_widgets_script_handlers( $elements_data );

        $depended_scripts = array_unique( $this->depended_scripts );

        $scripts_data = [
            'depends' => array_map( function ( $handle ) {
                $script_obj = $this->get_registered_script_obj_by_handler( $handle );

                return [
                    'handle' => $handle,
                    'src'    => $script_obj->src,
                    'obj'    => $script_obj
                ];
            }, $depended_scripts ),
            'widgets' => array_unique( $this->elementor_widgets ),
        ];

        update_post_meta( $popup_id, '_is_script_deps', $scripts_data );

        return $scripts_data;
    }

    /**
     * Get content style data
     *
     * @param $popup_id
     * @return array|array[]
     */
    public function get_content_style_data( $popup_id = false ) {

        if ( ! $popup_id ) {
            return [
                'depends' => [],
            ];
        }

        $is_deps = $this->get( 'is_style_deps', true );

        if ( ! $is_deps ) {
            return [
                'depends' => [],
            ];
        }

        $data_cache = get_post_meta( $popup_id, '_is_style_deps', true );

        if ( ! empty( $data_cache )  ) {
            return $data_cache;
        }

        $depended_styles = array_unique( $this->depended_styles );


        $styles_data = [
            'depends' => array_map( function ( $handle ) {
                $style_obj = $this->get_registered_style_obj_by_handler( $handle );

                return [
                    'handle' => $handle,
                    'src'    => $style_obj->src,
                    'obj'    => $style_obj
                ];
            }, $depended_styles ),
        ];

        update_post_meta( $popup_id, '_is_style_deps', $styles_data );

        return $styles_data;
    }

    /**
     * @return array
     */
    public function get_render_data() {
        $popup_id = $this->get( 'popup_id', false );

        $template_content = $this->get_content();

        $styles_data = $this->get_content_style_data( $popup_id );
        $scripts_data = $this->get_content_script_data( $popup_id );

        $template_styles = $styles_data['depends'];
        $template_scripts = $scripts_data['depends'];

        return [
            'content'       => $template_content,
            'styles'        => $template_styles,
            'scripts'       => $template_scripts,
            'afterScripts'  => [],
        ];
    }

}