<?php
namespace Phox_Host\Builder\Popup\Render;

use function Phox_Host\phox_host_core;
use Elementor\Icons_Manager;

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}


class Render_Manager{

    /**
     * @var array
     */
    public $defined_popup_list = [];

    /**
     * @var array
     */
    public $styles_enqueue = [];

    /**
     * @var array
     */
    public $scripts_enqueue = [];

    public function __construct(){
        $this->load_files();

        add_action( 'wp_footer', [ $this, 'init' ], 8 );
        add_action( 'wp_footer', [ $this, 'page_render' ], 9 );


    }

    public function load_files(){
        require __DIR__ .  DIRECTORY_SEPARATOR . 'base_render.php';
        require __DIR__ . DIRECTORY_SEPARATOR .'modules' . DIRECTORY_SEPARATOR . 'front_content.php';
    }

    /**
     * Init
     *
     * Get the popup that have condition to run and then add to it default settings
     *
     * @return false|void
     */
    public function init(){

        if( ! defined( 'ELEMENTOR_VERSION' ) || ! empty( $_GET['elementor-preview'] ) ){
            return false;
        }

        $condition_popups = phox_host_core()->theme_builder_condition->find_matched_by_conditions('wdes_popup');

        if ( empty( $condition_popups ) ) {
            return false;
        }

        $condition_popups = array_unique( $condition_popups );

        $this->defined_popup_list = array_map(function ( $popup_id ){
            $popup_settings = $this->get_settings( $popup_id );

            return [
                'id'       => $popup_id,
                'settings' => $popup_settings,
            ];
        }, $condition_popups);


    }

    public function page_render(){

        $defined_popup_list = $this->defined_popup_list;

        if ( empty( $defined_popup_list ) ) {
            return false;
        }

        foreach ( $defined_popup_list as $popup_id => $popup_data ){
            $this->popup_render($popup_data['id'], $popup_data['settings']);

        }

        $this->enqueue_styles();

        $this->enqueue_scripts();

    }

    /**
     * Get settings
     *
     * @param $popup_id
     * @return array
     */
        public function get_settings($popup_id){

        $default_settings = [
            'wdes_popup_settings_animation'             => 'fade',
            'wdes_popup_settings_open_target'           => 'attach',
            'wdes_popup_settings_open_delay'            => 1,
            'wdes_popup_settings_user_inactivity_time'  => 3,
            'wdes_popup_scrolled_to_value'              => 10,
            'wdes_popup_on_date_value'                  => '',
            'wdes_popup_on_time_start_value'            => '',
            'wdes_popup_on_time_end_value'              => '',
            'wdes_popup_prevent_scrolling'              => false,
            'wdes_popup_show_once'                      => false,
            'use_close_button'                          => true,
            'close_button_icon'                         => ['value' => 'fas fa-times', 'library' => 'fa-solid'],
            'use_overlay'                               => true,
            'close_on_overlay_click'                    => true,
            'wdes_popup_show_again_delay'               => 'none'
        ];

        $settings = get_post_meta($popup_id, '_elementor_page_settings', true);

        if ( ! empty( $settings ) ) {

            $popup_settings = [];

            foreach ( $default_settings as $option => $value ) {

                if ( array_key_exists( $option, $settings ) ) {
                    $popup_settings[ $option ] = $settings[ $option ];

                    $new_setting = 'selected_' . $option;

                    if ( array_key_exists( $new_setting, $settings ) ) {
                        $popup_settings[ $new_setting ] = $settings[ $new_setting ];
                    }

                } else {
                    $popup_settings[ $option ] = $value;
                }
            }

            return $popup_settings;

        }

        return $default_settings;

    }

    /**
     * Popup Render
     *
     * @param $popup_id
     * @param $popup_settings
     * @return false|void
     */
    public function popup_render( $popup_id = false, $popup_settings = [] ){

        if( empty( $popup_settings ) ){
            return false;
        }

        $close_button_html = '';
        $use_close_button = !isset($popup_settings['use_close_button']) || filter_var($popup_settings['use_close_button'], FILTER_VALIDATE_BOOLEAN);

        if( isset( $popup_settings['close_button_icon'] ) && $use_close_button ){
            $close_button_html = sprintf('<div class="wdes-popup-close-button modal__close" aria-label="Close modal" data-micromodal-close>%s</div>', $this->render_icon($popup_settings['close_button_icon'],['data-micromodal-close' => 'true']));
        }

        $overlay_html = '';
        $use_overlay = !isset($popup_settings['use_overlay']) || filter_var($popup_settings['use_overlay'], FILTER_VALIDATE_BOOLEAN);

        if( $use_overlay ){
            $close_overlay = filter_var( $popup_settings['close_on_overlay_click'], FILTER_VALIDATE_BOOLEAN ) ? 'data-micromodal-close' : '' ;
            $overlay_html = sprintf('<div class="wdes-popup-overlay modal__overlay" %s tabindex="-1"></div>', $close_overlay);
        }

        $show_again_delay =$this->switch_time_to_millisecond( $popup_settings['wdes_popup_show_again_delay'] );

        $popup_json = [
            'id'                      => $popup_id,
            'wdes-popup-id'           => 'wdes-popup-' . $popup_id,
            'animation'               => $popup_settings['wdes_popup_settings_animation'],
            'open-trigger'            => $popup_settings['wdes_popup_settings_open_target'],
            'page-load-delay'         => $popup_settings['wdes_popup_settings_open_delay'],
            'user-inactivity-time'    => $popup_settings['wdes_popup_settings_user_inactivity_time'],
            'scrolled-to'             => $popup_settings['wdes_popup_scrolled_to_value'],
            'on-date'                 => $popup_settings['wdes_popup_on_date_value'],
            'on-time-start'           => $popup_settings['wdes_popup_on_time_start_value'],
            'on-time-end'             => $popup_settings['wdes_popup_on_time_end_value'],
            'prevent-scrolling'       => filter_var( $popup_settings['wdes_popup_prevent_scrolling'], FILTER_VALIDATE_BOOLEAN ),
            'show-once'               => filter_var( $popup_settings['wdes_popup_show_once'], FILTER_VALIDATE_BOOLEAN ),
            'show-again-delay'        => $show_again_delay,
        ];

        $popup_json_data = htmlspecialchars(json_encode($popup_json));

        $class_list = [
            'wdes-popup',
            'wdes-popup-front-mode',
            'modal',
            'micromodal-slide',
            'wdes-popup-animation-'. $popup_settings['wdes_popup_settings_animation']
        ];

        $dependencies = get_post_meta( $popup_id, '_is_deps_ready', true );

        $is_content   = empty( $dependencies ) ? true : $dependencies;

         $render_instance = new \Phox_Host\Builder\Popup\Render\Modules\Front_Content([
             'popup_id'   => $popup_id,
             'with_css'   => true,
             'is_content' => $is_content
         ]);


        if ( $is_content ) {
            update_post_meta( $popup_id, '_is_deps_ready', 'true' );
        }

        $render_data = $render_instance->get_render_data();

        $popup_styles  = $render_data['styles'];
        $popup_scripts = $render_data['scripts'];


        $this->styles_enqueue = wp_parse_args( $popup_styles, $this->styles_enqueue );
        $this->scripts_enqueue = wp_parse_args( $popup_scripts, $this->scripts_enqueue );

        $content_html = sprintf( '<div class="wdes-popup-container-content">%1$s</div>', $render_data['content'] );
        $container_html = sprintf( '<div class="wdes-popup-container modal__container" role="dialog" aria-modal="true"><div class="wdes-popup-container-inner"><div class="wdes-popup-container-overlay"></div>%1$s</div>%2$s</div>',
            $content_html,
            $close_button_html
        );

        $html = sprintf( '<div id="wdes-popup-%1$s" class="%2$s" data-settings="%3$s" aria-hidden="false"><div class="wdes-popup-inner">%4$s%5$s</div></div>',
            $popup_id,
            implode( ' ', $class_list ),
            $popup_json_data,
            $overlay_html,
            $container_html
        );

        echo $html;

    }

    /**
     * Enqueue Styles
     *
     * @return false|void
     */
    public function enqueue_styles(){

       if( empty( $this->styles_enqueue ) ){
            return false;
       }

        foreach ( $this->styles_enqueue as $data ){
            $handle = $data['handle'];

            if( wp_style_is($handle) ){
                continue;
            }

            $obj = $data['obj'];

            if( ! isset( wp_styles()->registered[$handle] ) ){
                wp_styles()->registered[$handle] =$obj;
            }

            wp_enqueue_style( $obj->handle, $obj->src, $obj->deps, $obj->ver );

        }

    }

    /**
     * Enqueue Script
     *
     * @return false|void
     */
    public function enqueue_scripts(){

        if( empty( $this->scripts_enqueue ) ){
            return false;
        }

        foreach ( $this->scripts_enqueue as $data ){

            $handle = $data['handle'];

            if( wp_script_is($handle) ){
                continue;
            }

            $obj = $data['obj'];

            if( ! isset( wp_scripts()->registered[$handle] ) ){
                wp_scripts()->registered[$handle] =$obj;
            }

            wp_enqueue_script( $obj->handle, $obj->src, $obj->deps, $obj->ver );
            wp_scripts()->print_extra_script($obj->handle);

        }
    }

    /**
     * Switch Time To Millisecond
     *
     * @param $time
     * @return float|int|string
     */
    public function switch_time_to_millisecond( $time = 'none' ){

        if ( 'none' == $time ){
            return 'none';
        }

        switch ( $time ) {

            case 'minute':
                $delay = MINUTE_IN_SECONDS * 1000;
                break;

            case '10minutes':
                $delay = 10 * MINUTE_IN_SECONDS * 1000;
                break;

            case '30minutes':
                $delay = 30 * MINUTE_IN_SECONDS * 1000;
                break;

            case 'hour':
                $delay = HOUR_IN_SECONDS * 1000;
                break;

            case '3hours':
                $delay = 3 * HOUR_IN_SECONDS * 1000;
                break;

            case '6hours':
                $delay = 6 * HOUR_IN_SECONDS * 1000;
                break;

            case '12hours':
                $delay = 12 * HOUR_IN_SECONDS * 1000;
                break;

            case 'day':
                $delay = DAY_IN_SECONDS * 1000;
                break;

            case '3days':
                $delay = 3 * DAY_IN_SECONDS * 1000;
                break;

            case 'week':
                $delay = WEEK_IN_SECONDS * 1000;
                break;

            case 'month':
                $delay = MONTH_IN_SECONDS * 1000;
                break;

            default:
                $delay = 'none';
                break;
        }

        return $delay;

    }

    /**
     * Render Icon
     *
     * @param array $icon
     * @param array $attributes
     * @param string $tag
     *
     * @return mixed|string
     */
    public function render_icon( $icon, $attributes = [], $tag = 'i'){
        if ( empty( $icon['library'] ) ) {
            return false;
        }

        $output = '';

        /**
         * When the library value is svg it means that it's a SVG media attachment uploaded by the user.
         * Otherwise, it's the name of the font family that the icon belongs to.
         */
        if ( 'svg' === $icon['library'] ) {
            $output = Icons_Manager::render_uploaded_svg_icon( $icon['value'] );
        } else {
            $output = Icons_Manager::render_font_icon( $icon, $attributes, $tag );
        }

        return $output;
    }

}