<?php
namespace Phox_Host\Builder;

use function Phox_Host\phox_host_core;

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}

/**
 * Theme Builder Post Type
 *
 * Register and configure theme builder post type
 *
 * @package Builder
 * @since 2.0.0
 */
class Post_Type extends \Phox_Host\Builder\Base\Post_Type {

    /**
     * Post type slug.
     *
     * @var string
     */
    public $slug = 'wdes-theme-builder';


    /**
     * Constructor for the class
     */
    public function __construct() {

        new Structures();

        add_action( 'init', [$this, 'register_builder'] , 0);

        if ( is_admin() || (defined('WP_CLI') && WP_CLI) ) {
            add_action( 'admin_menu', [$this, 'add_builder_page'], 23 );
        }

        add_action( 'admin_action_wdes_create_new_template', [$this, 'create_template']);

        add_filter( 'manage_' . $this->slug . '_posts_columns', array( $this, 'set_post_columns' ) );

        add_action( 'manage_' . $this->slug . '_posts_custom_column', array( $this, 'post_columns' ), 10, 2 );

        parent::__construct();

    }

    /**
     * Register templates post type
     *
     * @since 2.0.0
     * @return void
     */
    public function register_builder(){

        $args = [
            'labels' => $this->register_post_type_labels(),
            'public'              => true,
            'hierarchical'        => false,
            'show_ui'             => true,
            'show_in_menu'        => false,
            'show_in_nav_menus'   => false,
            'can_export'          => true,
            'exclude_from_search' => true,
            'capability_type'     => 'post',
            'rewrite'             => false,
            'supports'            => ['title', 'editor', 'thumbnail', 'author', 'elementor'],
        ];

        register_post_type(
            $this->slug(),
            $args
        );

        $tax_args = array(
            'hierarchical'      => false,
            'show_ui'           => true,
            'show_in_nav_menus' => false,
            'show_admin_column' => true,
            'query_var'         => is_admin() || (defined('WP_CLI') && WP_CLI),
            'rewrite'           => false,
            'public'            => false,
            'label'             => __( 'Type', 'phox-host' ),
        );

        register_taxonomy(
            $this->type_tax,
            $this->slug(),
            $tax_args
        );

    }

    /**
     * add builder to admin menu
     *
     * @since 2.0.0
     * @return void
     */
    public function add_builder_page() {

        add_submenu_page(
            'phox',
            esc_html__( 'Theme Builder', 'phox-host' ),
            esc_html__( 'Theme Builder', 'phox-host' ),
            'edit_pages',
            'edit.php?post_type=' . $this->slug() . '&' . $this->type_tax . '=wdes_header'
        );

    }


    /**
     * Create new template
     *
     * @return [type] [description]
     */
    public function create_template() {

        if ( ! current_user_can( 'edit_posts' ) ) {
            wp_die(
                esc_html__( 'You don\'t have permissions to do this', 'phox-host' ),
                esc_html__( 'Error', 'phox-host' )
            );
        }

        $type = isset( $_REQUEST['template_type'] ) ? esc_attr( $_REQUEST['template_type'] ) : false;

        if ( ! $type || ! array_key_exists( $type, $this->tabs ) ) {
            wp_die(
                esc_html__( 'Incorrect template type. Please try again.', 'phox-host' ),
                esc_html__( 'Error', 'phox-host' )
            );
        }

        $documents = \Elementor\Plugin::instance()->documents;
        $doc_type  = $documents->get_document_type( $type );

        if ( ! $doc_type ) {
            wp_die(
                esc_html__( 'Incorrect template type. Please try again.', 'phox-host' ),
                esc_html__( 'Error', 'phox-host' )
            );
        }

        $post_data = array(
            'post_type'  => $this->slug,
            'meta_input' => array(
                '_elementor_edit_mode' => 'builder',
            ),
            'tax_input'  => array(
                $this->type_tax => $type,
            ),
            'meta_input' => array(
                $doc_type::TYPE_META_KEY   => $type,
                '_elementor_page_settings' => [],
            ),
        );

        $title = isset( $_REQUEST['template_name'] ) ? esc_attr( $_REQUEST['template_name'] ) : '';

        if ( $title ) {
            $post_data['post_title'] = $title;
        }

        $template_id = wp_insert_post( $post_data );

        if ( ! $template_id ) {
            wp_die(
                esc_html__( 'Can\'t create template. Please try again', 'phox-host' ),
                esc_html__( 'Error', 'phox-host' )
            );
        }

        if ( version_compare( ELEMENTOR_VERSION, '2.6.0', '<' ) ) {
            $redirect = \Elementor\Utils::get_edit_link( $template_id );
        } else {
            $redirect = \Elementor\Plugin::$instance->documents->get( $template_id )->get_edit_url();
        }

        wp_redirect( $redirect );
        die();

    }

    /**
     * Set required post columns
     *
     * @param array $columns
     * @return array
     */
    public function set_post_columns(array $columns ): array {

        unset( $columns['taxonomy-' . $this->type_tax ] );
        unset( $columns['date'] );

        $columns['type'] = __( 'Type', 'phox-host' );

        $columns['conditions'] = __( 'Active Conditions', 'phox-host' );

        $columns['date'] = __( 'Date', 'phox-host' );


        return $columns;

    }

    /**
     * Manage post columns content
     *
     * @param string $column
     * @param int $post_id
     * @return void
     */
    public function post_columns(string $column , int $post_id): void {

        $init_structures = new Structures();

        $structure = $init_structures->get_post_structure( $post_id );

        switch ( $column ) {

            case 'type':

                if ( $structure ) {

                    $link = add_query_arg( [
                        $this->type_tax => $structure['id'],
                    ]);

                    printf( '<a href="%1$s">%2$s</a>', $link, $structure['single_label'] );

                }

            break;

            case 'conditions':

                print ('<div class="wdes-builder-conditions">');

                printf(
                    '<div class="wdes-builder-conditions-list">%1$s</div>',
                    phox_host_core()->theme_builder_condition->post_conditions_verbose($post_id)
                );

                print ('</div>');

            break;

        }

    }

}