<?php
namespace Phox_Host\Elementor\Widgets;

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}
use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Controls_Manager;
use Elementor\Control_Media;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Typography;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Phox_Host\Elementor\Base\Base_Widget;

/**
 * Portfolio widget.
 *
 * Tabs widget that will display in Phox category
 *
 * @package Elementor\Widgets
 * @since 1.8.0
 */
class Portfolio extends Base_Widget {

    /**
     * Get Widget name
     *
     * Retrieve Portfolio widget name
     *
     * @return string Widget name
     * @since  1.8.0
     * @access public
     *
     */
    public function get_name() {
        return 'wdes_portfolio';
    }

    /**
     * Get Widget title
     *
     * Retrieve Portfolio widget title
     *
     * @return string Widget title
     * @since  1.8.0
     * @access public
     *
     */
    public function get_title() {
        return __( 'Portfolio', 'phox-host' );
    }

    /**
     * Get Widget icon
     *
     * Retrieve Portfolio widget icon
     *
     * @return string Widget icon
     * @since  1.8.0
     * @access public
     *
     */
    public function get_icon() {
        return 'wdes-widget-elementor wdes-widget-portfolio';
    }


    /**
     * Get Widget keywords
     *
     * Retrieve the list of keywords the widget belongs to.
     *
     * @return array widget keywords.
     * @since  1.8.0
     * @access public
     *
     */
    public function get_keywords() {
        return [ 'portfolio', 'gallery'];
    }

    /**
     * Get script dependencies.
     *
     * Used to set scripts dependencies required to run the widget.
     *
     * @return array Widget scripts dependencies.
     * @since 1.8.0
     * @access public
     *
     */
    public function get_script_depends() {
        return [ 'wdes-masonry','wdes-anime', 'imagesloaded' ];
    }

    /**
     * Register Tabs widget controls
     *
     * Adds different input fields to allow the user to change and customize the widget settings.
     *
     * @since  1.8.0
     * @access protected
     */
    protected function register_controls() {

        $css_scheme = [
            'list_container'   => '.wdes-portfolio-list',
            'inner'            => '.wdes-portfolio-inner',
            'image_instance'   => '.wdes-portfolio-image-instance',
            'content_inner'    => '.wdes-portfolio-content-inner',
            'cover'            => '.wdes-portfolio-cover',
            'title'            => '.wdes-portfolio-title',
            'desc'             => '.wdes-portfolio-desc',
            'category'         => '.wdes-portfolio-category',
            'button'           => '.wdes-portfolio-button',
            'filters_wrap'     => '.wdes-portfolio-filter',
            'filters'          => '.wdes-portfolio-filter-list',
            'filter'           => '.wdes-portfolio-filter-item',
            'filter_separator' => '.wdes-portfolio-filter-item-separator',
        ];

        $this->start_controls_section(
            'section_items_data',
            [
                'label' => esc_html__( 'Items', 'phox-host' ),
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'item_title',
            [
                'label'   => esc_html__( 'Title', 'phox-host' ),
                'type'    => Controls_Manager::TEXT,
                'dynamic' => ['active' => true],
            ]
        );

        $repeater->add_control(
            'item_desc',
            [
                'label'   => esc_html__( 'Description', 'phox-host' ),
                'type'    => Controls_Manager::TEXTAREA,
                'dynamic' => ['active' => true],
            ]
        );

        $repeater->add_control(
            'item_image',
            [
                'label'   => esc_html__( 'Image', 'phox-host' ),
                'type'    => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
                'dynamic' => ['active' => true],
            ]
        );

        $repeater->add_control(
            'item_category',
            [
                'label'   => esc_html__( 'Category', 'phox.host' ),
                'type'    => Controls_Manager::TEXT,
                'dynamic' => ['active' => true],
            ]
        );

        $repeater->add_control(
            'item_button_text',
            [
                'label'   => esc_html__( 'Link Text', 'phox-host' ),
                'type'    => Controls_Manager::TEXT,
                'default' => esc_html__( 'More', 'phox-host' ),
            ]
        );

        $repeater->add_control(
            'item_button_url',
            [
                'label'       => esc_html__( 'Link Url', 'phox-host' ),
                'type'        => Controls_Manager::URL,
                'placeholder' => 'https://example.com',
                'default' => [
                    'url' => '',
                ],
                'dynamic' => ['active' => true],
            ]
        );

        $repeater->add_control(
            'item_image_link',
            [
                'label'   => esc_html__( 'Item Link', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'lightbox',
                'options' => [
                    'lightbox' => esc_html__( 'Lightbox', 'phox-host' ),
                    'external' => esc_html__( 'External', 'phox-hosts' ),
                ],
            ]
        );

        $this->add_control(
            'image_list',
            [
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $repeater->get_controls(),
                'default'     => [
                    [
                        'item_title'       => esc_html__( 'Item 1', 'phox-host' ),
                        'item_desc'        => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'phox-host' ),
                    ],
                    [
                        'item_title'       => esc_html__( 'Item 2', 'phox-host' ),
                        'item_desc'        => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'phox-host' ),
                    ],
                    [
                        'item_title'       => esc_html__( 'Item 3', 'phox-host' ),
                        'item_desc'        => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'phox-host' ),
                    ],
                    [
                        'item_title'       => esc_html__( 'Item 4', 'phox-host' ),
                        'item_desc'        => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'phox-host' ),
                    ],
                    [
                        'item_title'       => esc_html__( 'Item 5', 'phox-host' ),
                        'item_desc'        => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'phox-host' ),
                    ],
                    [
                        'item_title'       => esc_html__( 'Item 6', 'phox-host' ),
                        'item_desc'        => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.', 'phox-host' ),
                    ],
                ],
                'title_field' => '{{{ item_title }}}',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_layout_settings',
            [
                'label' => esc_html__( 'Layout Setting', 'phox-host' ),
            ]
        );

        $this->add_control(
            'layout_type',
            [
                'label'   => esc_html__( 'Layout Type', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'masonry',
                'options' => [
                    'masonry' => esc_html__( 'Masonry', 'phox-host' ),
                    'grid' => esc_html__( 'Grid', 'phox-hosts' ),
                    'justify' => esc_html__( 'Justify', 'phox-hosts' ),
                    'list' => esc_html__( 'List', 'phox-hosts' ),
                ],
            ]
        );

        $this->add_control(
            'layout_preset',
            [
                'label'   => esc_html__( 'Layout Presets', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'type-1',
                'options' => [
                    'type-1' => esc_html__( 'Image With Text Box When Hover', 'phox-host' ),
                    'type-2' => esc_html__( 'Image With Text Below', 'phox-hosts' ),
                    'type-3' => esc_html__( 'Only Image', 'phox-hosts' ),
                    'type-4' => esc_html__( 'Image With Full Text Box When Hover', 'phox-hosts' ),
                ],
            ]
        );

        $this->add_responsive_control(
            'columns',
            [
                'label'   => esc_html__( 'Columns', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 3,
                'options' => [
                    1 => 1,
                    2 => 2,
                    3 => 3,
                    4 => 4,
                    5 => 5,
                    6 => 6,
                ],
                'condition' => [
                    'layout_type' => ['masonry', 'grid'],
                ]
            ]
        );

        $this->add_responsive_control(
            'item_margin',
            [
                'label' => esc_html__( 'Items Gap', 'phox-host' ),
                'type'  => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 50,
                    ],
                ],
                'default' => [
                    'size' => 10,
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['inner']          => 'margin: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} ' . $css_scheme['list_container'] => 'margin: -{{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'item_height',
            [
                'label' => esc_html__( 'Item Height', 'phox-host' ),
                'type'  => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => 100,
                        'max' => 1000,
                    ],
                ],
                'default' => [
                    'size' => 300,
                ],
                'condition' => [
                    'layout_type' => [
                        'grid',
                        'justify',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image_instance'] => 'height: {{SIZE}}{{UNIT}};',
                ]
            ]
        );

        $this->add_control(
            'cover_icon',
            [
                'label'       => esc_html__( 'Cover Icon', 'phox-host' ),
                'type'        => Controls_Manager::ICONS,
                'label_block' => true,
                'default' => [
                    'value' => 'fas fa-search-plus',
                    'library' => 'fa-solid',
                ],
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3',
                    ],
                ]
            ]
        );

        $this->add_control(
            'title_html_tag',
            [
                'label'     => esc_html__('HTML Tag', 'phox-host'),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'h1'         =>'H1',
                    'h2'         =>'H2',
                    'h3'         =>'H3',
                    'h4'         =>'H4',
                    'h5'         =>'H5',
                    'h6'         =>'H6',
                    'div'        =>'DIV',
                    'span'       =>'Span',
                    'p'          =>'P'
                ],
                'default'   => 'h4',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_filter_settings',
            [
                'label' => esc_html__( 'Filter Setting', 'phox-host' ),
            ]
        );

        $this->add_control(
            'enable_filter',
            array(
                'label'        => esc_html__( 'Enable Filter', 'phox-host' ),
                'type'         => Controls_Manager::SWITCHER,
                'label_on'     => esc_html__( 'Yes', 'phox-host' ),
                'label_off'    => esc_html__( 'No', 'phox-host' ),
                'return_value' => 'true',
                'default'      => false,
            )
        );

        $this->add_control(
            'all_filter_label',
            [
                'label'   => esc_html__( '`All` Filter Label', 'phox-host' ),
                'type'    => Controls_Manager::TEXT,
                'default' => esc_html__( 'All', 'phox-host' ),
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->add_control(
            'separator_icon',
            [
                'label'       => esc_html__( 'Separator Icon', 'phox-host' ),
                'type'        => Controls_Manager::ICONS,
                'label_block' => true,
                'default' => [
                    'value' => 'fas fa-hashtag',
                    'library' => 'fa-solid',
                ],
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->end_controls_section();

        /**
         * Style Section
         */

        /**
         * Filter Container Section
         */

        $this->start_controls_section(
                'section_portfolio_filter_container_style',
            [
                'label'      => esc_html__( 'Filter Container', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'filters_container_background',
                'selector' => '{{WRAPPER}} ' . $css_scheme['filters'] ,
            ]
        );

        $this->add_responsive_control(
            'filters_container_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filters'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'filters_container_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filters'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'filters_container_border',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filters'],
            ]
        );

        $this->add_responsive_control(
            'filters_container_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filters'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'filters_box_shadow',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filters'],
            ]
        );

        $this->end_controls_section();

        /**
         * Filter Items Section
         */

        $this->start_controls_section(
            'section_portfolio_filter_items_style',
            [
                'label'      => esc_html__( 'Filter Items', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_responsive_control(
            'filters_items_aligment',
            [
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Start', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-left' : 'eicon-h-align-right',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-h-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'End', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-right' : 'eicon-h-align-left',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['filters_wrap'] => 'justify-content: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'filter_items_separator_color',
            [
                'label' => esc_html__( 'Separator Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['filter_separator'] => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_control(
            'filter_items_separator_size',
            [
                'label' => esc_html__( 'Separator Size', 'phox-host' ),
                'type'       => Controls_Manager::SLIDER,
                'range'      => [
                    'px' => [
                        'min' => 5,
                        'max' => 500,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['filter_separator'] => 'font-size: {{SIZE}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'filter_items_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'filters_items_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->start_controls_tabs( 'tabs_filter_item' );

        $this->start_controls_tab(
            'tab_filter_item_normal',
            [
                'label' => esc_html__( 'Normal', 'phox-host' ),
            ]
        );

        $this->add_control(
            'filter_color',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'filter_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['filter'],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'filter_background',
                'selector' => '{{WRAPPER}} ' . $css_scheme['filter'] ,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'filter_border',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filter'],
            ]
        );

        $this->add_responsive_control(
            'filter_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'filter_shadow',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filter'],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'tab_filter_item_hover',
            [
                'label' => esc_html__( 'Hover', 'phox-host' ),
            ]
        );
        $this->add_control(
            'filter_color_hover',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] . ':hover' => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'filter_typography_hover',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['filter'] . ':hover',
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'filter_background_hover',
                'selector' => '{{WRAPPER}} ' . $css_scheme['filter'] . ':hover' ,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'filter_border_hover',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filter'] . ':hover',
            ]
        );

        $this->add_responsive_control(
            'filter_border_radius_hover',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] . ':hover' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'filter_shadow_hover',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filter'] . ':hover',
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'tab_filter_item_active',
            [
                'label' => esc_html__( 'Active', 'phox-host' ),
            ]
        );

        $this->add_control(
            'filter_color_active',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] . '.active' => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'filter_typography_active',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['filter'] . '.active',
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'filter_background_active',
                'selector' => '{{WRAPPER}} ' . $css_scheme['filter'] . '.active',
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'filter_border_active',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filter'] . '.active',
            ]
        );

        $this->add_responsive_control(
            'filter_border_radius_active',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['filter'] . '.active' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'filter_shadow_active',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['filter'] . '.active',
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

        /**
         * General Style
         */
        $this->start_controls_section(
            'section_portfolio_general_style',
            [
                'label'      => esc_html__( 'Item', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_responsive_control(
            'item_padding',
            [
                'label'      => __( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['inner'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'image_border',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['inner'],
            ]
        );

        $this->add_responsive_control(
            'image_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['inner'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'image_shadow',
                'exclude' => [
                    'box_shadow_position',
                ],
                'selector'    => '{{WRAPPER}} ' . $css_scheme['inner'],
            ]
        );

        $this->add_control(
            'image_overlay_heading',
            [
                'label'      => esc_html__( 'Overlay', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'image_overlay_icon_background',
                'selector' => '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon',
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_control(
            'image_overlay_icon_font_color',
            [
                'label' => esc_html__( 'Font Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon' => 'color: {{VALUE}}',
                    '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon  svg' => 'fill: {{VALUE}}',
                ],
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_control(
            'image_overlay_icon_font_size',
            [
                'label' => esc_html__( 'Font Size', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [
                    'px', 'em', 'rem',
                ],
                'responsive' => true,
                'range'      => [
                    'px' => [
                        'min' => 5,
                        'max' => 500,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon' => 'font-size: {{SIZE}}{{UNIT}}',
                    '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon:before' => 'font-size: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_control(
            'image_overlay_icon_box_size',
            [
                'label' => esc_html__( 'Box Size', 'phox-host' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [
                    'px', 'em', 'rem',
                ],
                'responsive' => true,
                'range'      => [
                    'px' => [
                        'min' => 5,
                        'max' => 500,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                ],
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'image_overlay_icon_border',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon',
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'image_overlay_icon_shadow',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['cover'] . ' .wdes-elements-icon',
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'image_overlay_background',
                'selector' => '{{WRAPPER}} ' . $css_scheme['cover'] ,
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_responsive_control(
            'image_overlay_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['cover'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->add_responsive_control(
            'image_overlay_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['cover'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition' => [
                    'layout_preset' => [
                        'type-2',
                        'type-3'
                    ]
                ]
            ]
        );

        $this->end_controls_section();

        /**
         * Content Section
         */

        $this->start_controls_section(
            'section_portfolio_filter_content_style',
            [
                'label'      => esc_html__( 'Content', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_control(
            'section_portfolio_content_wrapper_heading',
            [
                'label'      => esc_html__( 'Container', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
            ]
        );

        $this->add_responsive_control(
            'content_container_alignment',
            [
                'label'   => esc_html__( 'Content Position', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Start', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-left' : 'eicon-h-align-right',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-h-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'End', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-right' : 'eicon-h-align-left',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['content_inner'] => 'justify-content: {{VALUE}};',
                ],
                'condition' => [
                    'layout_preset' => [
                        'type-4'
                    ]
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'content_container_background',
                'selector' => '{{WRAPPER}} .preset-type-2 ' . $css_scheme['inner'] . ', {{WRAPPER}} ' . $css_scheme['content_inner'],
            ]
        );

        $this->add_responsive_control(
            'content_wrapper_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['content_inner'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'content_wrapper_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'default'    => [ 'top' => 10 , 'right' => 10, 'left' => 10, 'bottom' => 10],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['content_inner'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'content_wrapper_border',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['content_inner'],
            ]
        );

        $this->add_responsive_control(
            'content_wrapper_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['content_inner'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'content_wrapper_box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['content_inner'],
            ]
        );

        $this->add_control(
            'section_portfolio_image_heading',
            [
                'label'      => esc_html__( 'Image', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_responsive_control(
            'image_content_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['image_instance'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_control(
            'section_portfolio_title_heading',
            [
                'label'      => esc_html__( 'Title', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_responsive_control(
            'title_alignment',
            [
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Start', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-left' : 'eicon-h-align-right',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-h-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'End', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-right' : 'eicon-h-align-left',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'text-align: {{VALUE}};',
                ]
            ]
        );

        $this->add_control(
            'title_color',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'title_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['title'],
            ]
        );

        $this->add_responsive_control(
            'title_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'title_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_control(
            'section_portfolio_category_heading',
            [
                'label'      => esc_html__( 'Category', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->add_responsive_control(
            'category_alignment',
            [
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Start', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-left' : 'eicon-h-align-right',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-h-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'End', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-right' : 'eicon-h-align-left',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['category'] => 'text-align: {{VALUE}};',
                ],
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->add_control(
            'category_color',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['category'] => 'color: {{VALUE}}',
                ],
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'category_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['category'],
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->add_responsive_control(
            'category_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['category'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->add_responsive_control(
            'category_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['category'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition' => [
                    'enable_filter' => 'true',
                ]
            ]
        );

        $this->add_control(
            'section_portfolio_desc_heading',
            [
                'label'      => esc_html__( 'Description', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'desc_alignment',
            [
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Start', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-left' : 'eicon-h-align-right',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-h-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'End', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-right' : 'eicon-h-align-left',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['desc'] => 'text-align: {{VALUE}};',
                ]
            ]
        );

        $this->add_control(
            'desc_color',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['desc'] => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'desc_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['desc'],

            ]
        );

        $this->add_responsive_control(
            'desc_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['desc'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'desc_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['desc'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_control(
            'section_portfolio_button_heading',
            [
                'label'      => esc_html__( 'Button', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'button_alignment',
            [
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Start', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-left' : 'eicon-h-align-right',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-h-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'End', 'phox-host' ),
                        'icon'  => ! is_rtl() ? 'eicon-h-align-right' : 'eicon-h-align-left',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'align-self: {{VALUE}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'button_text_alignment',
            [
                'label'   => esc_html__( 'Text Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'text-align: {{VALUE}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'button_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'button_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->start_controls_tabs( 'tabs_button_style' );

        $this->start_controls_tab(
            'tab_button_normal',
            [
                'label' => esc_html__( 'Normal', 'phox-host' ),
            ]
        );

        $this->add_control(
            'button_color',
            [
                'label' => esc_html__( 'Text Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_control(
            'button_bg_color',
            [
                'label'     => esc_html__( 'Background Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'background-color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'button_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_ACCENT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['button'],

            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'button_border',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['button'],
            ]
        );

        $this->add_responsive_control(
            'button_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'button_box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['button'],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'tab_button_hover',
            [
                'label' => esc_html__( 'Hover', 'phox-host' ),
            ]
        );

        $this->add_control(
            'button_color_hover',
            [
                'label' => esc_html__( 'Text Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['button'] . ':hover' => 'color: {{VALUE}}',
                ]
            ]
        );

        $this->add_control(
            'button_bg_color_hover',
            [
                'label'     => esc_html__( 'Background Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['button'] . ':hover' => 'background-color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'button_typography_hover',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_ACCENT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['button'] . ':hover',

            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'  => 'button_border_hover',
                'label' => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default' => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['button'] . ':hover',
            ]
        );

        $this->add_responsive_control(
            'button_border_radius_hover',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['button'] . ':hover' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'button_box_shadow_hover',
                'selector' => '{{WRAPPER}} ' . $css_scheme['button'] . ':hover',
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs( );

        $this->add_control(
            'section_portfolio_order_heading',
            [
                'label'      => esc_html__( 'Order', 'phox-host' ),
                'type'       => Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_control(
            'title_order',
            [
                'label'   => esc_html__( 'Title Order', 'phox-host' ),
                'type'    => Controls_Manager::NUMBER,
                'default' => 1,
                'min'     => 1,
                'max'     => 4,
                'step'    => 1,
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['title'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'category_order',
            [
                'label'   => esc_html__( 'Category Order', 'phox-host' ),
                'type'    => Controls_Manager::NUMBER,
                'default' => 2,
                'min'     => 1,
                'max'     => 4,
                'step'    => 1,
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['category'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'desc_order',
            [
                'label'   => esc_html__( 'Description Order', 'phox-host' ),
                'type'    => Controls_Manager::NUMBER,
                'default' => 3,
                'min'     => 1,
                'max'     => 4,
                'step'    => 1,
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['desc'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'button_order',
            [
                'label'   => esc_html__( 'Button Order', 'phox-host' ),
                'type'    => Controls_Manager::NUMBER,
                'default' => 4,
                'min'     => 1,
                'max'     => 4,
                'step'    => 1,
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['button'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_portfolio_lightbox_style',
            [
                'label'      => esc_html__( 'Lightbox', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'container_lightbox_background',
                'selector' => '.elementor-slideshow__header',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Render
     *
     * @since  1.8.0
     * @access protected
     */
    protected function render() {
        $settings = $this->get_settings_for_display();

        $general_setting = $this->get_general_settings($settings);

        $images_list = [];

        $categories = [];

        foreach ($settings['image_list'] as $item){

            $image_item = $item['item_image'];

            $alt = esc_attr( Control_Media::get_image_alt( $image_item ) );

            if( ! empty( $id = $image_item['id'] ) ){

                $image_data = wp_get_attachment_image_src( $id, 'full' );

                $item['item_image']['url'] = $image_data[0];
                $item['item_image']['width'] = $image_data[1];
                $item['item_image']['height'] = $image_data[2];


            }else{

                $item['item_image']['url'] = $image_item['url'];
                $item['item_image']['width'] = 1200;
                $item['item_image']['height'] = 800;

            }

            $item['item_image']['alt'] = $alt;

            if( ! empty( $category = $item['item_category'] ) ){

                if( ! in_array($category, $categories) ){

                    if( $category === 'All' ){

                        $category = '&ensp;' . $category;

                    }

                    $categories[] = $category;

                }

            }

            $images_list [] = $item;

        }

        $images_json = json_encode( $images_list );

        $categories_json = json_encode( $categories );

        $data_settings = 'data-portfolio-settings="'.esc_attr($general_setting) .'"' ;

        $data_image = 'data-portfolio-images="'.esc_attr($images_json) .'"' ;

        $data_categories = 'data-portfolio-categories="'.esc_attr($categories_json) .'"' ;

        //classes
        $classes = [ 'wdes-portfolio' ,'layout-type-'.$settings['layout_type'], 'preset-'.$settings['layout_preset'] ];

        if( $settings['layout_type'] == 'masonry' || $settings['layout_type'] == 'grid' ){

            $classes [] = 'layout-desktop-column-' . $settings['columns'];

            $classes [] = ! empty( $settings['columns_tablet'] ) ? 'layout-tablet-column-' . $settings['columns_tablet'] : false;

            $classes [] = ! empty( $settings['columns_mobile'] ) ? 'layout-mobile-column-' . $settings['columns_mobile'] : false;

        }

        printf ('<div class = "%s" %s >', implode(' ',$classes) ,$data_settings);

        if( ! empty($categories) ){

            printf('<div class="wdes-portfolio-filter" %s></div>', $data_categories);

        }

        printf('<div class="wdes-portfolio-list" %s></div>', $data_image);

        print ('</div>');
    }

    public function get_general_settings($settings){

        $result = [];

        //layout
        $result['layout']['type']   = $settings['layout_type'];
        $result['layout']['preset'] = $settings['layout_preset'];
        $result['layout']['coverIcon']['value'] = (!empty($settings['cover_icon']['value'])) ? $settings['cover_icon']['value'] : null;
        $result['layout']['coverIcon']['library'] = (!empty($settings['cover_icon']['library'])) ? $settings['cover_icon']['library'] : null;
        $result['layout']['titleHtmlTag'] = $settings['title_html_tag'];
        $result['layout']['columns'] = $settings['layout_preset'];
        ! empty( $settings['columns_tablet'] ) ? $result['layout']['columnsTablet'] = $settings['columns_tablet'] : false;
        ! empty( $settings['columns_mobile'] ) ? $result['layout']['columnsMobile'] = $settings['columns_mobile'] : false;

        //filter
        $result['filter']['enable']   = (bool) $settings['enable_filter'];
        $result['filter']['label']   = $settings['all_filter_label'];
        $result['filter']['separatorIcon']['value']   = (!empty($settings['separator_icon']['value'])) ? $settings['separator_icon']['value'] : null;
        $result['filter']['separatorIcon']['library']   = (!empty($settings['separator_icon']['library'])) ? $settings['separator_icon']['library'] : null;
        return json_encode( $result );

    }


}