<?php
namespace Phox_Host\Elementor\Widgets;

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}

use Elementor\Control_Media;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Utils;
use Elementor\Repeater;
use Phox\helpers;
use Phox_Host\Elementor\Base\Base_Widget;

/**
 * Range Slider widget.
 *
 * Range Slider widget that will display in Phox category
 *
 * @package Elementor\Widgets
 * @since 1.8.1
 */
class Range_Slider extends Base_Widget {
    /**
     * Get Widget name
     *
     * Retrieve Range Slider widget name
     *
     * @since  1.8.1
     * @access public
     *
     * @return string Widget name
     */
    public function get_name() {
        return 'wdes_range_slider';
    }

    /**
     * Get Widget Title
     *
     * Retrieve Range Slider widget title
     *
     * @return string Widget title
     * @since  1.8.1
     * @access public
     *
     */
    public function get_title() {
        return __( 'Range Slider', 'phox-host' );
    }

    /**
     * Get Widget icon
     *
     * Retrieve Range Slider widget icon
     *
     * @return string Widget icon
     * @since  1.8.1
     * @access public
     *
     */
    public function get_icon() {
        return 'wdes-widget-elementor wdes-widget-range-slider';
    }

    /**
     * Get style dependencies.
     *
     * Retrieve the list of style dependencies the element requires.
     *
     * @since 1.8.1
     * @access public
     *
     * @return array Element styles dependencies.
     */
    public function get_style_depends() {
        return [ 'wp-jquery-ui-slider' ,'wdes-slider-pips' ];
    }

    /**
     * Get script dependencies.
     *
     * Used to set scripts dependencies required to run the widget.
     *
     * @return array Widget scripts dependencies.
     * @since 1.8.1
     * @access public
     *
     */
    public function get_script_depends() {
        return [ 'jquery-ui-slider','wdes-slider-pips' ];
    }


    /**
     * Get Widget keywords
     *
     * Retrieve the list of keywords the widget belongs to.
     *
     * @return array widget keywords.
     * @since  1.8.1
     * @access public
     *
     */
    public function get_keywords() {
        return [ 'slider', 'price' ];
    }


    /**
     * Register Range Slider controls
     *
     * Adds different input fields to allow the user to change and customize the widget settings.
     *
     * @since  1.8.1
     * @access protected
     */
    protected function register_controls() {

        $css_scheme =[
            'instance'        => '.wdes-range-slider',
            'content_wrapper' => '.slider-content',
            'content'         => '.slider-content-item',
            'slider_bar'      => '.slider-tiers',
            'active_bar'      => '.ui-slider-range',
            'handle'          => '.ui-slider-handle',
            'label'           => '.ui-slider-label',
            'title'           => '.wdes-range-slider-label',
            'description'     => '.wdes-range-slider-description',
            'image'           => '.wdes-range-slider-image',
            'icon'            => '.wdes-range-slider-icon',
            'active'          => '.ui-slider-pip-selected'
        ];

        $this->start_controls_section(
            'tab_items_section',
            [
                'label' => esc_html__( 'Content', 'phox-host' ),
            ]
        );

        $range_items = new Repeater();

        $range_items->add_control(
            'range_label',
            [
                'label'       => esc_html__( 'Title', 'phox-host' ),
                'type'        => Controls_Manager::TEXT,
                'default'     => esc_html__( 'Range Tier Title', 'phox-host' ),
                'dynamic'     => [
                    'active' => true,
                ],
                'label_block' => true,
            ]
        );

        $range_items->add_control(
            'range_description',
            [
                'label'       => esc_html__( 'Description', 'phox-host' ),
                'type'        => Controls_Manager::TEXTAREA,
                'dynamic'     => [
                    'active' => true,
                ],
                'label_block' => true,
            ]
        );

        $range_items->add_control(
            'text_decoration',
            [
                'label'   => __( 'Above Title', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'options' => [
                    'image' => [
                        'title' => esc_html__( 'Image', 'phox-host' ),
                        'icon'  => 'eicon-image',
                    ],
                    'icon'  => [
                        'title' => esc_html__( 'Icon', 'phox-host' ),
                        'icon'  => 'eicon-font-awesome',
                    ],
                ],
                'default' => 'icon',
            ]
        );

        $range_items->add_control(
            'range_icon',
            [
                'label'       => esc_html__( 'Icon', 'phox-host' ),
                'type'        => Controls_Manager::ICONS,
                'label_block' => true,
                'file'        => '',
                'skin' => 'inline',
                'exclude_inline_options' => ['svg'],
                'default'     => [
                    'value' => 'fas fa-arrows-alt'
                ],
                'condition'   => [
                    'text_decoration' => 'icon',
                ],
            ]
        );

        $range_items->add_control(
            'range_image',
            [
                'label'     => esc_html__( 'Image', 'phox-host' ),
                'type'      => Controls_Manager::MEDIA,
                'condition' => [
                    'text_decoration' => 'image',
                ],
            ]
        );

        $range_items->add_control(
            'content_type',
            [
                'label'       => esc_html__( 'Content Type', 'phox-host' ),
                'type'        => Controls_Manager::SELECT,
                'default'     => 'editor',
                'options'     => [
                    'template' => esc_html__( 'Template', 'phox-host' ),
                    'editor'   => esc_html__( 'Editor', 'phox-host' ),
                ],
                'label_block' => 'true',
            ]
        );

        $range_items->add_control(
            'range_editor_content',
            [
                'label'     => esc_html__( 'Content', 'phox-host' ),
                'type'      => Controls_Manager::WYSIWYG,
                'default'   => esc_html__( 'Range Tier Content', 'phox-host' ),
                'dynamic'   => [
                    'active' => true,
                ],
                'condition' => [
                    'content_type' => 'editor',
                ],
            ]
        );

        $range_items->add_control(
            'range_template_content',
            [
                'label'       => esc_html__( 'Templates', 'phox-host' ),
                'type'        => 'wdes-select-templates',
                'default'     => 'select-template',
                'condition'   => [
                    'content_type' => 'template',
                ],
                'label_block' => 'true',
            ]
        );

        $this->add_control(
            'range_items',
            [
                'label'       => __( 'Items', 'phox-host' ),
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $range_items->get_controls(),
                'default'     => [
                    [
                        'range_label' => esc_html__( 'Tier #1', 'phox-host' ),
                    ],
                    [
                        'range_label' => esc_html__( 'Tier #2', 'phox-host' ),
                    ],
                    [
                        'range_label' => esc_html__( 'Tier #3', 'phox-host' ),
                    ],
                ],
                'title_field' => '{{{range_label}}}',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_general_style',
            [
                'label'      => esc_html__( 'General', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );
        
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'range_container_background',
                'selector' => '{{WRAPPER}} ' . $css_scheme['instance'],
            ]
        );

        $this->add_responsive_control(
            'range_container_padding',
            [
                'label'      => __( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['instance'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_container_margin',
            [
                'label'      => __( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['instance'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'        => 'range_container_border',
                'label'       => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default'     => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['instance'],
            ]
        );

        $this->add_responsive_control(
            'range_container_border_radius',
            [
                'label'      => __( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['instance'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'     => 'range_container_box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['instance'],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_slider_style',
            [
                'label'      => esc_html__( 'Slider', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_control(
            'range_slider_bar',
            [
                'label'	=> esc_html__('Bar', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'range_slider_background_bar',
                'selector' => '{{WRAPPER}} ' . $css_scheme['slider_bar'],
            ]
        );

        $this->add_responsive_control(
            'range_slider_height_bar',
            [
                'label'       => esc_html__('Height', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['slider_bar'] => 'height:{{SIZE}}{{UNIT}}'
                ]
            ]
        );


        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'           => 'range_slider_border_bar',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'selector'       => '{{WRAPPER}} ' . $css_scheme['slider_bar'],
            ]
        );

        $this->add_responsive_control(
            'range_slider_radius_bar',
            [
                'label'      => __( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['slider_bar'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'     => 'range_slider_box_shadow_bar',
                'selector' => '{{WRAPPER}} ' . $css_scheme['slider_bar'],
            ]
        );

        $this->add_responsive_control(
            'range_slider_margin_bar',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'default'    => [ 'top' => 0, 'right' => 0, 'bottom' => 3.4, 'left' => 0, 'unit' => 'em', 'isLinked' => true ],
                'size_units' => ['em','px', '%'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['slider_bar'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_active_bar',
            [
                'label'	=> esc_html__('Active bar', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_control(
            'range_slider_active_bar_bg_color',
            [
                'label'  => esc_html__( 'Background Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['active_bar'] => 'background: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_active_bar_radius',
            [
                'label'      => __( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['active_bar'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_handler',
            [
                'label'	=> esc_html__('Handler', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_control(
            'range_slider_background_handler',
            [
                'label'  => esc_html__( 'Background Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['handle'] => 'background: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_height_handler',
            [
                'label'       => esc_html__('Height', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['handle'] => 'height:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'range_slider_width_handler',
            [
                'label'       => esc_html__('Width', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['handle'] => 'width:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'           => 'range_slider_border_handler',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'selector'       => '{{WRAPPER}} ' . $css_scheme['handle'],
            ]
        );

        $this->add_responsive_control(
            'range_slider_radius_handler',
            [
                'label'      => __( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['handle'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'     => 'range_slider_box_shadow_handler',
                'selector' => '{{WRAPPER}} ' . $css_scheme['handle'],
            ]
        );

        $this->add_control(
            'range_slider_position_y_handler',
            [
                'label'	=> esc_html__('Y position', 'phox-host'),
                'type'	=> Controls_Manager::SLIDER,
                'size_units' => ['px', '%'],
                'selectors'	=>	[
                    '{{WRAPPER}} '.$css_scheme['handle']	=>	'top: {{SIZE}}{{UNIT}};'
                ]
            ]
        );

        $this->add_control(
            'range_slider_position_x_handler',
            [
                'label'	=> esc_html__('X position', 'phox-host'),
                'type'	=> Controls_Manager::SLIDER,
                'size_units' => ['px', '%'],
                'selectors'	=>	[
                    '{{WRAPPER}} '.$css_scheme['handle']	=>	'left: {{SIZE}}{{UNIT}};'
                ]
            ]
        );

        $this->add_control(
            'range_slider_label',
            [
                'label'	=> esc_html__('Label', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'range_slider_width_label',
            [
                'label'       => esc_html__('Width', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'default'       => [
                    'unit'  =>  'em',
                    'size'  =>  6
                ],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['label'] => 'width:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'range_slider_alignment_label',
            [
                'label'     => __('Alignment', 'phox-host'),
                'type'      => Controls_Manager::CHOOSE,
                'default'   => '',
                'options'   => [
                    'left'    => [
                        'title' =>  __('Left', 'phox-host'),
                        'icon'  =>  'eicon-arrow-left'
                    ],
                    'center'    => [
                        'title' =>  __('Center', 'phox-host'),
                        'icon'  =>  'eicon-text-align-justify'
                    ],
                    'right'    => [
                        'title' =>  __('Right', 'phox-host'),
                        'icon'  =>  'eicon-arrow-right'
                    ],
                ],
                'selectors'     => [
                    '{{WRAPPER}} ' . $css_scheme['label']    =>  'text-align: {{VALUE}};'
                ]
            ]
        );

        $this->add_responsive_control(
            'range_slider_margin_label',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'default'    => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => -3, 'unit' => 'em' ,'isLinked' => true ],
                'size_units' => ['em','px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['label'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_divider',
            [
                'label'	=> esc_html__('Title', 'phox-host'),
                'type'	=> Controls_Manager::DIVIDER
            ]
        );

        $this->start_controls_tabs( 'tabs_range_slider_style' );

        $this->start_controls_tab(
            'range_slider_normal',
            [
                'label' => esc_html__( 'Normal', 'phox-host' ),
            ]
        );

        $this->add_control(
            'range_slider_title',
            [
                'label'	=> esc_html__('Title', 'phox-host'),
                'type'	=> Controls_Manager::HEADING
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'range_slider_title_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['title'],
            ]
        );

        $this->add_control(
            'range_slider_title_color',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_title_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_desc',
            [
                'label'	=> esc_html__('Description', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'range_slider_desc_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['description'],
            ]
        );

        $this->add_control(
            'range_slider_desc_color',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['description'] => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_desc_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['description'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_image',
            [
                'label'	=> esc_html__('Image', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'range_slider_height_image',
            [
                'label'       => esc_html__('Height', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'height:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'range_slider_width_image',
            [
                'label'       => esc_html__('Width', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'width:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_control(
            'range_slider_icon',
            [
                'label'	=> esc_html__('Icon', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'range_slider_icon_size',
            [
                'label'      => esc_html__( 'Icon Size', 'phox-host' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [
                    'px',
                    'em',
                    'rem',
                ],
                'range'      => [
                    'px' => [
                        'min' => 18,
                        'max' => 200,
                    ],
                ],
                'selectors'  => [
                    '{{WRAPPER}} '.$css_scheme['icon'] => 'font-size: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        $this->add_control(
            'range_slider_icon_color',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['icon'] => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'range_slider_hover',
            [
                'label' => esc_html__( 'Hover', 'phox-host' ),
            ]
        );

        $this->add_control(
            'range_slider_title_hover',
            [
                'label'	=> esc_html__('Title', 'phox-host'),
                'type'	=> Controls_Manager::HEADING
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'range_slider_title_typography_hover',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['title'] . ':hover',
            ]
        );

        $this->add_control(
            'range_slider_title_color_hover',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['title'] . ':hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_title_margin_hover',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['title'] . ':hover' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_desc_hover',
            [
                'label'	=> esc_html__('Description', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'range_slider_desc_typography_hover',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['description'] . ':hover',
            ]
        );

        $this->add_control(
            'range_slider_desc_color_hover',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['description'] . ':hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_desc_margin_hover',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['description'] . ':hover' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_image_hover',
            [
                'label'	=> esc_html__('Image', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'range_slider_height_image_hover',
            [
                'label'       => esc_html__('Height', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] . ':hover' => 'height:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'range_slider_width_image_hover',
            [
                'label'       => esc_html__('Width', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] . ':hover' => 'width:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_control(
            'range_slider_icon_hover',
            [
                'label'	=> esc_html__('Icon', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'range_slider_icon_size_hover',
            [
                'label'      => esc_html__( 'Icon Size', 'phox-host' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [
                    'px',
                    'em',
                    'rem',
                ],
                'range'      => [
                    'px' => [
                        'min' => 18,
                        'max' => 200,
                    ],
                ],
                'selectors'  => [
                    '{{WRAPPER}} '.$css_scheme['icon'] . ':hover' => 'font-size: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        $this->add_control(
            'range_slider_icon_color_hover',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['icon'] . ':hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'range_slider_active',
            [
                'label' => esc_html__( 'Active', 'phox-host' ),
            ]
        );

        $this->add_control(
            'range_slider_title_active',
            [
                'label'	=> esc_html__('Title', 'phox-host'),
                'type'	=> Controls_Manager::HEADING
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'range_slider_title_typography_active',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['title'] ,
            ]
        );

        $this->add_control(
            'range_slider_title_color_active',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['active'] . ' ' . $css_scheme['title'] => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_title_margin_active',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['title']  => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_desc_active',
            [
                'label'	=> esc_html__('Description', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'range_slider_desc_typography_active',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['description'],
            ]
        );

        $this->add_control(
            'range_slider_desc_color_active',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['description'] => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_slider_desc_margin_active',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['description']  => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'range_slider_image_active',
            [
                'label'	=> esc_html__('Image', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'range_slider_height_image_active',
            [
                'label'       => esc_html__('Height', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['image']  => 'height:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'range_slider_width_image_active',
            [
                'label'       => esc_html__('Width', 'phox-host'),
                'type'        => Controls_Manager::SLIDER,
                'size_units'  => ['px', 'em', '%'],
                'range'       => [
                    '%' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'em' => [
                        'min'  => 1,
                        'max'  => 100,
                        'step' => 1
                    ],
                    'px' => [
                        'min'  => 1,
                        'max'  => 1600,
                        'step' => 1
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['image'] => 'width:{{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_control(
            'range_slider_icon_active',
            [
                'label'	=> esc_html__('Icon', 'phox-host'),
                'type'	=> Controls_Manager::HEADING,
                'separator' => 'before'
            ]
        );

        $this->add_responsive_control(
            'range_slider_icon_size_active',
            [
                'label'      => esc_html__( 'Icon Size', 'phox-host' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => [
                    'px',
                    'em',
                    'rem',
                ],
                'range'      => [
                    'px' => [
                        'min' => 18,
                        'max' => 200,
                    ],
                ],
                'selectors'  => [
                    '{{WRAPPER}} '. $css_scheme['active'] . ' ' . $css_scheme['icon'] => 'font-size: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        $this->add_control(
            'range_slider_icon_color_active',
            [
                'label'  => esc_html__( 'Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['active'] . ' ' . $css_scheme['icon'] => 'color: {{VALUE}}',
                ],
            ]
        );
        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

        /**
         * Tabs Content Style Section
         */
        $this->start_controls_section(
            'section_range_content_style',
            [
                'label'      => esc_html__( 'Content', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_responsive_control(
            'control_wrapper_width',
            [
                'label'     => esc_html__( 'Control Width', 'phox-host' ),
                'type'      => Controls_Manager::SLIDER,
                'range'     => [
                    'min' => 0,
                    'max' => 100,
                ],
                'condition' => [
                    'range_position' => [ 'left', 'right' ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['content_wrapper'] => 'width: {{SIZE}}%',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'     => 'range_content_background',
                'selector' => '{{WRAPPER}} ' . $css_scheme['content_wrapper'],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'range_content_editor_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['content'],
            ]
        );
        $this->add_control(
            'range_content_editor_color',
            [
                'label'     => esc_html__( 'Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'default'   => '#708791',
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['content'] => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'range_content_padding',
            [
                'label'      => __( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'default'    => [ 'top' => 20, 'right' => 20, 'bottom' => 20, 'left' => 20, 'isLinked' => true ],
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['content'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'           => 'range_content_border',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'fields_options' => [
                    'border' => [
                        'default' => 'solid',
                    ],
                    'width'  => [
                        'default' => [
                            'top'      => '1',
                            'right'    => '1',
                            'bottom'   => '1',
                            'left'     => '1',
                            'isLinked' => false,
                        ],
                    ],
                    'color'  => [
                        'default' => '#e7e7e7',
                    ],
                ],
                'selector'       => '{{WRAPPER}} ' . $css_scheme['content_wrapper'],
            ]
        );

        $this->add_responsive_control(
            'range_content_radius',
            [
                'label'      => __( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['content_wrapper'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'     => 'range_content_box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['content_wrapper'],
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Render button widget output on the frontend.
     *
     * Written in PHP and used to generate the final HTML.
     *
     * @since 1.8.1
     * @access protected
     */
    protected function render() {

        $settings = $this->get_settings_for_display();

        $range_tiers_settings = [];

        foreach ( $settings['range_items'] as $item ){

            $collection = [];

            $text_decoration = $item['text_decoration'];

            $collection ['label'] = esc_html($item['range_label']);

            $collection ['desc'] = esc_html($item['range_description']);

            $collection ['textDecoration'] = esc_html($item['text_decoration']);

            if( $text_decoration === 'icon' ){

                $collection ['icon'] = $item['range_icon'];

            }  else{

                $collection ['icon'] = [];

            }

            if( $text_decoration === 'image' ){

                $collection ['image']['url'] = $item['range_image']['url'];

            }else{

                $collection ['image'] = [];

            }

            $range_tiers_settings [] = $collection;

        }

        $data_range_tiers = 'data-range-tiers="'.esc_attr(json_encode($range_tiers_settings)) .'"' ;

        printf ( '<div class="wdes-range-slider" %s>', $data_range_tiers);

            print('<div class="slider-tiers"></div>');

            print('<div class="slider-content">');

                foreach ( $settings['range_items'] as $key => $item ){

                    $class = ( $key === 0 ) ? 'd-block' : 'd-none' ;

                    printf('<div class="slider-content-item %s">',$class );
                    $content_html = '';

                    if ( 'template' === $item['content_type'] ) {

                        $template_id = $item['range_template_content'];

                        if ( get_post_type( $template_id ) === 'elementor_library' ) {

                            $template_shortcode = $this->elementor()->frontend->get_builder_content_for_display( $template_id, true );

                            $content_html .= $template_shortcode;

                        } else {

                            $content_html .= ' Please Select The Template ';

                        }
                    } else {

                        $content_html .= $this->parse_text_editor( $item['range_editor_content'] );

                    }

                    echo $content_html ;

                    print ('</div>');

                }

            print('</div>');

        print('</div>');

    }
}