<?php
namespace Phox_Host\Elementor\Widgets;

use Elementor\Controls_Manager;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Css_Filter;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Repeater;
use Phox\helpers;
use Phox\helpers as Helper;
use Phox_Host\Elementor\Base\Base_Widget;
use Phox_Host\Elementor\Functions\WHMCS\Products;

if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}

/**
 * WHMCS Price Table Widget.
 *
 * WHMCS Price Table widget that will display in Phox category
 *
 * @package Elementor\Widgets
 * @since 1.4.1
 */
class WHMCS_Price_Table extends Base_Widget {
    /**
     * WHMCS Price Table constructor.
     *
     * Initializing the WHMCS price table class.
     *
     * @access public
     *
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     */
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        $this->products_list = get_option('wdes_whmcs_fetch_data');

        $this->set_first_product_id();
    }

    /**
     * Products Handler
     * @since  1.4.1
     */
    use Products;

    /**
     * Get Widget name
     *
     * Retrieve Tabs widget name
     *
     * @since  1.4.1
     * @access public
     *
     * @return string Widget name
     */
    public function get_name() {
        return 'wdes_whmcs_price_table';
    }

    /**
     * Get Widget title
     *
     * Retrieve Tabs widget title
     *
     * @since  1.4.1
     * @access public
     *
     * @return string Widget title
     */
    public function get_title() {
        return __( 'WHMCS Price Table', 'phox-host' );
    }

    /**
     * Get Widget icon
     *
     * Retrieve Tabs widget icon
     *
     * @since  1.4.1
     * @access public
     *
     * @return string Widget icon
     */
    public function get_icon() {
        return 'wdes-widget-elementor wdes-widget-whmcs-pricing';
    }


    /**
     * Get Widget keywords
     *
     * Retrieve the list of keywords the widget belongs to.
     *
     * @since  1.4.1
     * @access public
     *
     * @return array widget keywords.
     */
    public function get_keywords() {
        return [ 'price', 'table', 'product', 'image', 'plan', 'button' ];
    }

    /**
     * Register Tabs widget controls
     *
     * Adds different input fields to allow the user to change and customize the widget settings.
     *
     * @since  1.4.1
     * @access protected
     */
    protected function register_controls() {

        $css_scheme = [
            'table'                     => '.wdes-pricing-table',
            'header'                    => '.wdes-pricing-table-heading',
            'icon_wrap'                 => '.wdes-pricing-table-icon',
            'icon_box'                  => '.wdes-pricing-table-icon-box',
            'icon'                      => '.wdes-pricing-table-icon-box > *',
            'image'                     => '.wdes-pricing-table-img',
            'title'                     => '.wdes-pricing-table-title',
            'subtitle'                  => '.wdes-pricing-table-subtitle',
            'price'                     => '.wdes-pricing-table-price',
            'price_prefix'              => '.wdes-pricing-table-price-perfix',
            'price_discount'            => '.wdes-pricing-table-price-original',
            'price_value'               => '.wdes-pricing-table-price-num',
            'price_suffix'              => '.wdes-pricing-table-price-suffix',
            'price_desc'                => '.wdes-pricing-table-price-desc',
            'features'                  => '.wdes-pricing-table-features',
            'features_wrap'             => '.wdes-pricing-table-features-wrap',
            'features_header'           => '.wdes-pricing-table-features-header',
            'features_item'             => '.wdes-pricing-feature',
            'feature_item'              => '.wdes-pricing-feature-inner',
            'feature_item_text'         => '.wdes-pricing-table-features-text',
            'feature_item_description_button'   => '.wdes-pricing-feature-inner .extra-info .extra-info-trigger',
            'feature_item_description_content'  => '.wdes-pricing-feature-inner .extra-info .extra-info-content',
            'footer'                    => '.wdes-pricing-feature-footer',
            'button'                    => '.wdes-pricing-feature-footer .pricing-table-button',
            'button_icon'               => '.wdes-pricing-feature-footer .pricing-table-button-icon',
            'strip_default_main'        => '.wdes-pricing-table-strip-default',
            'strip_default'             => '.wdes-pricing-table-strip-default-inner',
            'strip_custom'              => '.wdes-pricing-table-strip-custom-image',
        ];


        $this->start_controls_section(
            'section_whmcs_setting',
            [
                'label' => esc_html__( 'General Settings', 'phox-host' ),
            ]
        );

        $this->add_control(
            'whmcs_settings_select_product',
            [
                'label' => esc_html__( 'Select Product', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'default' => $this->first_product_id,
                'options' => $this->get_products_names()
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_header',
            [
                'label' => esc_html__( 'Header', 'phox-host' ),
            ]
        );

        foreach ($this->get_products_names() as $id => $item){

            $this->add_control(
                'title_'.$id,
                [
                    'label'     => esc_html__( 'Title', 'phox-host' ),
                    'type'      => Controls_Manager::TEXT,
                    'default'   => $item,
                    'dynamic'   => [ 'active' => true ],
                    'condition'   => [
                        'whmcs_settings_select_product' => $id,
                    ],
                ]
            );

        }

        $this->add_control(
            'title_html_tag',
            [
                'label'     => esc_html__('Title HTML Tag', 'phox-host'),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'h1'         =>'H1',
                    'h2'         =>'H2',
                    'h3'         =>'H3',
                    'h4'         =>'H4',
                    'h5'         =>'H5',
                    'h6'         =>'H6',
                    'div'        =>'DIV',
                    'span'       =>'Span',
                    'p'          =>'P'
                ],
                'default'   => 'h2',
            ]
        );

        $this->add_control(
            'subtitle',
            [
                'label'     => esc_html__( 'Subtitle', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
                'default'   => esc_html__( 'Subtitle', 'phox-host' ),
                'dynamic'   => [ 'active' => true ],
            ]
        );

        $this->add_control(
            'above_title',
            [
                'label'     => __( 'Above Title', 'phox-host' ),
                'type'      => Controls_Manager::CHOOSE,
                'options'   => [
                    'image' => [
                        'title' => esc_html__( 'Image', 'phox-host' ),
                        'icon'  => 'eicon-image',
                    ],
                    'icon' => [
                        'title' => esc_html__( 'Icon', 'phox-host' ),
                        'icon'  => 'eicon-font-awesome',
                    ],
                ],
                'default'   => 'icon',
            ]
        );

        $this->add_control(
            'icon',
            [
                'label'       => esc_html__( 'Icon', 'phox-host' ),
                'type'        => Controls_Manager::ICONS,
                'label_block' => true,
                'default' => [
                    'value' => 'far fa-hdd',
                    'library' => 'fa-regular',
                ],
                'condition'     => [
                    'above_title'   => 'icon',
                ],
            ]
        );

        $this->add_control(
            'image',
            [
                'label' => esc_html__( 'Image', 'phox-host' ),
                'type'  => Controls_Manager::MEDIA,
                'condition'     => [
                    'above_title'   => 'image',
                ],
            ]
        );

        $this->add_control(
            'strip',
            [
                'label'        => esc_html__( 'Strip', 'phox-host' ),
                'type'         => Controls_Manager::SWITCHER,
                'label_on'     => esc_html__( 'Yes', 'phox-host' ),
                'label_off'    => esc_html__( 'No', 'phox-host' ),
                'return_value' => 'yes',
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'price_strip_type',
            [
                'type'      => Controls_Manager::CHOOSE,
                'options'   => [
                    'strip_default' => [
                        'title' => esc_html__( 'Default', 'phox-host' ),
                        'icon'  => 'eicon-frame-minimize',
                    ],
                    'strip_custom' => [
                        'title' => esc_html__( 'Custom', 'phox-host' ),
                        'icon'  => 'eicon-image',
                    ],
                ],
                'default'   => 'strip_default',
                'condition'     => [
                    'strip'   => 'yes',
                ],
            ]
        );

        $this->add_control(
            'strip_default_title',
            [
                'label'     => esc_html__( 'Title', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
                'default'   => esc_html__( 'Popular', 'phox-host' ),
                'condition'     => [
                    'strip'   => 'yes',
                    'price_strip_type'  => 'strip_default',
                ],
            ]
        );

        $this->add_control(
            'strip_custom_image',
            [
                'label' => esc_html__( 'Image', 'phox-host' ),
                'type'  => Controls_Manager::MEDIA,
                'condition'     => [
                    'strip'   => 'yes',
                    'price_strip_type'  => 'strip_custom',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_price',
            [
                'label' => esc_html__( 'Price', 'phox-host' ),
            ]
        );

        $this->add_control(
            'currency_price',
            [
                'label'     => esc_html__( 'Currency', 'phox-host' ),
                'type'      => Controls_Manager::SELECT,
                'default'   => 'USD',
                'options'   => $this->get_product_currencies()
            ]
        );

        $this->add_control(
            'payment_type',
            [
                'label'     => esc_html__( 'Billing Cycle', 'phox-host' ),
                'type'      => Controls_Manager::SELECT,
                'default'   => 'monthly',
                'options'   => [
                    'monthly'       => esc_html__( 'Monthly', 'phox-host' ),
                    'quarterly'     => esc_html__( 'Quarterly', 'phox-host' ),
                    'semiannually'  => esc_html__( 'Semi-Annually', 'phox-host' ),
                    'annually'      => esc_html__( 'Annually', 'phox-host' ),
                    'biennially'    => esc_html__( 'Biennially', 'phox-host' ),
                    'triennially'   => esc_html__( 'Triennially', 'phox-host' ),
                ]
            ]
        );

        $this->add_control(
            'discount',
            [
                'label'     => esc_html__( 'Discount', 'phox-host' ),
                'type'      => Controls_Manager::SWITCHER,
                'label_on'  => __( 'On', 'phox-host' ),
                'label_off'  => __( 'OFF', 'phox-host' ),
            ]
        );

        $this->add_control(
            'original_price',
            [
                'label'     => esc_html__( 'Original Price', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
                'default'   => esc_html__( '39.99', 'phox-host' ),
                'condition'     => [
                    'discount'   => 'yes',
                ],

            ]
        );

        foreach ($this->currencies as $currency ){
            $this->add_control(
                'perfix_'.$currency,
                [
                    'label'     => esc_html__( 'Prefix', 'phox-host' ),
                    'type'      => Controls_Manager::TEXT,
                    'default'   => $this->get_currency_prefix($currency),
                    'condition'   => [
                          'currency_price' => $currency,
                    ],
                ]
            );
        }


        $this->add_control(
            'suffix',
            [
                'label'     => esc_html__( 'Suffix', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
                'default'   => esc_html__( '/per month', 'phox-host' ),
                'dynamic'   => [ 'active' => true ],
            ]
        );

        $this->add_control(
            'desc',
            [
                'label'     => esc_html__( 'Description', 'phox-host' ),
                'type'      => Controls_Manager::TEXTAREA,
                'dynamic'   => [ 'active' => true ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_feature',
            [
                'label' => esc_html__( 'Features', 'phox-host' ),
            ]
        );

        $this->add_control(
            'feature_header',
            [
                'label'     => esc_html__( 'Feature Header', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
                'default'   => esc_html__( ' Head Title ', 'phox-host' ),
                'label_block'   => true,
            ]
        );

        $feature_items = new Repeater();

        $feature_items->add_control(
            'item_title',
            [
                'label'     => esc_html__( 'Title', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
                'default'   => esc_html__( 'List Item', 'phox-host' ),
                'label_block'   => true,
            ]
        );

        $feature_items->add_control(
            'item_description',
            [
                'label'     => esc_html__( 'Description', 'phox-host' ),
                'type'      => Controls_Manager::WYSIWYG,
                'default'   => '',
                'label_block'   => true,
            ]
        );

        $feature_items->add_control(
            'item_description_icon',
            [
                'label'       => esc_html__( 'Description Icon', 'phox-host' ),
                'type'        => Controls_Manager::ICONS,
                'skin' => 'inline',
                'exclude_inline_options' => ['svg'],
                'default'     => [
                    'value'   => 'far fa-question-circle',
                    'library' => 'fa-regular',
                ],
                'label_block' => true,
                'condition'     => [
                    'item_description!'   => '',
                ]
            ]
        );

        $feature_items->add_control(
            'item_decoration',
            [
                'label'     => __( 'Text Decoration', 'phox-host' ),
                'type'      => Controls_Manager::CHOOSE,
                'options'   => [
                    'image' => [
                        'title' => esc_html__( 'Image', 'phox-host' ),
                        'icon'  => 'eicon-image',
                    ],
                    'icon' => [
                        'title' => esc_html__( 'Icon', 'phox-host' ),
                        'icon'  => 'eicon-font-awesome',
                    ],
                ],
                'default'   => 'icon',
            ]
        );

        $feature_items->add_control(
            'item_icon',
            [
                'label'         => esc_html__( 'Icon', 'phox-host' ),
                'type'          => Controls_Manager::ICONS,
                'label_block'   => true,
                'default' => [
                    'value' => 'fas fa-flag',
                    'library' => 'fa-solid',
                ],
                'condition'     => [
                    'item_decoration'   => 'icon',
                ],
            ]
        );

        $feature_items->add_control(
            'item_image',
            [
                'name'  => 'item_image',
                'label' => esc_html__( 'Image', 'phox-host' ),
                'type'  => Controls_Manager::MEDIA,
                'condition'     => [
                    'item_decoration'   => 'image',
                ],
            ]
        );

        foreach ( $this->get_products_description() as $upid => $item_desc ){
            $this->add_control(
                'feature_items_'.$upid,
                [
                    'label' => __( 'Tab Items', 'phox-host' ),
                    'type'  => Controls_Manager::REPEATER,
                    'fields' => $feature_items->get_controls(),
                    'default'  => $item_desc,
                    'title_field' => '{{{item_title}}}',
                    'condition'   => [
                        'whmcs_settings_select_product' => $upid,
                    ],

                ]
            );
        }


        $this->end_controls_section();

        $this->start_controls_section(
            'section_footer',
            [
                'label' => esc_html__( 'Footer', 'phox-host' ),
            ]
        );

        $this->add_control(
            'button_before',
            [
                'label'     => esc_html__( 'Text Before Button', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
            ]
        );

        $this->add_control(
            'button_text',
            [
                'label'     => esc_html__( 'Button Text', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
                'default'   => esc_html__( 'Buy', 'phox-host' ),
            ]
        );

        foreach ( $this->get_products_url() as $upid => $d_url ){
            $this->add_control(
                'button_link_'.$upid,
                [
                    'label'         => esc_html__( 'Link', 'phox-host' ),
                    'type'          => Controls_Manager::URL,
                    'placeholder'   => __( 'https://example.com', 'phox-host' ),
                    'default'       => $d_url,
                    'dynamic'       => [
                        'active'    => true,
                    ],
                    'condition'   => [
                        'whmcs_settings_select_product' => $upid,
                    ],
                ]
            );
        }


        $this->add_control(
            'button_after',
            [
                'label'     => esc_html__( 'Text After Button', 'phox-host' ),
                'type'      => Controls_Manager::TEXT,
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_table_style',
            array(
                'label'      => esc_html__( 'Table', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            )
        );

        $this->start_controls_tabs( 'table_bg_border' );

        $this->start_controls_tab(
            'table_bg_border_normal',
            array(
                'label' => esc_html__( 'Normal', 'phox-host' ),
            )
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            array(
                'name'     => 'table_bg',
                'selector' => '{{WRAPPER}} ' . $css_scheme['table'],
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'           => 'table_border',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'placeholder'    => '1px',
                'selector'       => '{{WRAPPER}} ' . $css_scheme['table'],
                'fields_options' => array(
                    'border' => array(
                        'default' => 'solid',
                    ),
                    'width' => array(
                        'default' => array(
                            'top'      => '1',
                            'right'    => '1',
                            'bottom'   => '1',
                            'left'     => '1',
                            'isLinked' => true,
                        ),
                    ),
                    'color' => array(
                        'global' => [
                            'default' => Global_Colors::COLOR_TEXT
                        ],
                        'default' => '#e7e7e7',
                    ),
                ),
            )
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'table_bg_border_hover',
            array(
                'label' => esc_html__( 'Hover', 'phox-host' ),
            )
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            array(
                'name'     => 'table_bg_hover',
                'selector' => '{{WRAPPER}} ' . $css_scheme['table'] . ':hover',
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'           => 'table_border_hover',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'placeholder'    => '1px',
                'selector'       => '{{WRAPPER}} ' . $css_scheme['table'] . ':hover',
                'fields_options' => array(
                    'border' => array(
                        'default' => 'solid',
                    ),
                    'width' => array(
                        'default' => array(
                            'top'      => '1',
                            'right'    => '1',
                            'bottom'   => '1',
                            'left'     => '1',
                            'isLinked' => true,
                        ),
                    ),
                    'color' => array(
                        'global' => [
                            'default' => Global_Colors::COLOR_TEXT
                        ],
                        'default' => '#e7e7e7',
                    ),
                ),
            )
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->add_responsive_control(
            'table_border_radius',
            [
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['table'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            array(
                'name'     => 'table_box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['table'],
            )
        );

        $this->add_responsive_control(
            'table_padding',
            array(
                'label'      => esc_html__( 'Table Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['table'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_control(
            'order_heading',
            [
                'label'     => esc_html__( 'Order', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'header_order',
            [
                'label'     => esc_html__( 'Header Order', 'phox-host' ),
                'type'      => Controls_Manager::NUMBER,
                'default'   => 1,
                'min'       => 1,
                'max'       => 4,
                'step'      => 1,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['header'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'price_order',
            [
                'label'     => esc_html__( 'Price Order', 'phox-host' ),
                'type'      => Controls_Manager::NUMBER,
                'default'   => 2,
                'min'       => 1,
                'max'       => 4,
                'step'      => 1,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['price'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'features_order',
            [
                'label'     => esc_html__( 'Features Order', 'phox-host' ),
                'type'      => Controls_Manager::NUMBER,
                'default'   => 3,
                'min'       => 1,
                'max'       => 4,
                'step'      => 1,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['features'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'footer_order',
            [
                'label'     => esc_html__( 'Footer Order', 'phox-host' ),
                'type'      => Controls_Manager::NUMBER,
                'default'   => 4,
                'min'       => 1,
                'max'       => 4,
                'step'      => 1,
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['footer'] => 'order: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_header_style',
            array(
                'label'      => esc_html__( 'Header', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            )
        );

        $this->add_control(
            'header_bg_color',
            array(
                'label' => esc_html__( 'Background Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['header'] => 'background-color: {{VALUE}}',
                ),
            )
        );

        $this->add_control(
            'header_title_style',
            array(
                'label'     => esc_html__( 'Title', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'title_color',
            array(
                'label'  => esc_html__( 'Title Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_SECONDARY
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'color: {{VALUE}}',
                ),
                'default' => '#1b3a4e',
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'title_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['title'],
            )
        );

        $this->add_responsive_control(
            'title_margin',
            [
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['title'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            ]
        );

        $this->add_control(
            'header_subtitle_style',
            array(
                'label'     => esc_html__( 'Subtitle', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'subtitle_color',
            array(
                'label'  => esc_html__( 'Subtitle Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_SECONDARY
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['subtitle'] => 'color: {{VALUE}}',
                ),
                'default' => '#708791',
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'subtitle_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['subtitle'],
            )
        );

        $this->add_responsive_control(
            'header_margin',
            array(
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}}  ' . $css_scheme['subtitle'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_control(
            'header_separator',
            [
                'type'  => Controls_Manager::DIVIDER,
                'style' => 'thick',
            ]
        );

        $this->add_responsive_control(
            'header_padding',
            array(
                'label'      => esc_html__( 'Header Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['header'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'header_alignment',
            array(
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => array(
                    'left'    => array(
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-arrow-left',
                    ),
                    'center' => array(
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-justify',
                    ),
                    'right' => array(
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-arrow-right',
                    ),
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['header'] => 'text-align: {{VALUE}};',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'           => 'header_border',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'placeholder'    => '1px',
                'selector'       => '{{WRAPPER}} ' . $css_scheme['header'],
            )
        );

        $this->add_responsive_control(
            'header_border_radius',
            array(
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['header'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_above_title_style',
            [
                'label'      => esc_html__( 'Above Title', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name'      => 'icon_box_border',
                'selector'  => '{{WRAPPER}} ' . $css_scheme['icon'],
                'condition'  => [
                    'above_title' => 'icon',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name'      => 'icon_box_background',
                'selector'  => '{{WRAPPER}} ' . $css_scheme['icon'],
                'condition'  => [
                    'above_title' => 'icon',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'      => 'icon_box_shadow',
                'selector'  => '{{WRAPPER}} ' . $css_scheme['icon'],
                'condition'  => [
                    'above_title' => 'icon',
                ],
            ]
        );

        $this->add_control(
            'icon_box_color', [
                'label'  => esc_html__( 'Icon Color', 'phox-host' ),
                'type'   => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_SECONDARY
                ],
                'condition'  => [
                    'above_title' => 'icon',
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['icon'] => 'color: {{VALUE}}',
                ],
                'default' => '#1b3a4e',
            ]
        );

        $this->add_responsive_control(
            'icon_box_font-size',
            [
                'label'   => esc_html__( 'Font Size', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', 'em' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 500,
                        'step' => 5,
                    ],
                    'em' => [
                        'min'  => 0,
                        'max'  => 10,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 40,
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['icon'] => 'font-size: {{SIZE}}{{UNIT}};',
                ],
                'condition'  => [
                    'above_title' => 'icon',
                ],
            ]
        );

        $this->add_responsive_control(
            'icon_box_width',
            [
                'label'   => esc_html__( 'Box Size', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 500,
                        'step' => 5,
                    ],
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 50,
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['icon'] => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition'  => [
                    'above_title' => 'icon',
                ],
            ]
        );

        $this->add_responsive_control(
            'icon_wrap_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['icon_wrap'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
                'separator' => 'before',
                'condition'  => [
                    'above_title' => 'icon',
                ],
            ]
        );

        $this->add_responsive_control(
            'icon_box_alignment',
            [
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => array(
                    'left'    => array(
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-arrow-left',
                    ),
                    'center' => array(
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-justify',
                    ),
                    'right' => array(
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-arrow-right',
                    ),
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['icon_wrap'] => 'text-align: {{VALUE}};',
                ),
                'condition'  => [
                    'above_title' => 'icon',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_above_title_width',
            [
                'label'   => esc_html__( 'Width', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1000,
                    ],
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_above_title_space',
            [
                'label'   => esc_html__( 'Max Width', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1000,
                    ],
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'max-width: {{SIZE}}{{UNIT}};',
                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_above_title_height',
            [
                'label'   => esc_html__( 'Height', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1000,
                    ],
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'height: {{SIZE}}{{UNIT}};',
                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_above_title_max_height',
            [
                'label'   => esc_html__( 'Max Height', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 1000,
                    ],
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'max-height: {{SIZE}}{{UNIT}};',
                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_control(
            'image_above_title_separator',
            [
                'type'  => Controls_Manager::DIVIDER,
                'style' => 'thick',
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->start_controls_tabs( 'image_effects' );
        $this->start_controls_tab(
            'normal',
            [
                'label' => esc_html__( 'Normal', 'phox-host' ),
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_control(
            'image_above_title_opacity',
            [
                'label' => __( 'Opacity', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'max' => 1,
                        'min' => 0.10,
                        'step' => 0.01,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'opacity: {{SIZE}};',
                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name' => 'image_above_title_css_filters',
                'selector' => '{{WRAPPER}} ' . $css_scheme['image'],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab( 'hover',
            [
                'label' => __( 'Hover', 'phox-host' ),
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_control(
            'image_above_title_opacity_hover',
            [
                'label' => __( 'Opacity', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'max' => 1,
                        'min' => 0.10,
                        'step' => 0.01,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] . ':hover' => 'opacity: {{SIZE}};',

                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name' => 'image_above_title_css_filters_hover',
                'selector' => '{{WRAPPER}} ' . $css_scheme['image'] . ':hover',
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_control(
            'image_above_title_background_hover_transition',
            [
                'label' => __( 'Transition Duration', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'max' => 3,
                        'step' => 0.1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'transition-duration: {{SIZE}}s',
                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_control(
            'image_above_title_hover_animation',
            [
                'label' => __( 'Hover Animation', 'phox-host' ),
                'type' => Controls_Manager::HOVER_ANIMATION,
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'image_above_title_border',
                'selector' => '{{WRAPPER}} ' . $css_scheme['image'],
                'separator' => 'before',
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_above_title_border_radius',
            [
                'label' => __( 'Border Radius', 'phox-host' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['image'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'image_above_title_box_shadow',
                'exclude' => [
                    'box_shadow_position',
                ],
                'selector' => '{{WRAPPER}} ' . $css_scheme['image'],
                'condition'  => [
                    'above_title' => 'image',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_pricing_style',
            array(
                'label'      => esc_html__( 'Pricing', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            )
        );

        $this->add_control(
            'price_bg_color',
            array(
                'label' => esc_html__( 'Background Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price'] => 'background-color: {{VALUE}}',
                ),
            )
        );

        $this->add_control(
            'price_prefix_style',
            array(
                'label'     => esc_html__( 'Prefix', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'price_prefix_color',
            array(
                'label' => esc_html__( 'Price Prefix Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'color: {{VALUE}}',
                ),
                'default' => '#1b3a4e',
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'price_prefix_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['price_prefix'],
            )
        );

        $this->add_control(
            'price_prefix_display',
            array(
                'label'   => esc_html__( 'Prefix Display', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'options' => array(
                    'inline-block' => esc_html__( 'Inline', 'phox-host' ),
                    'block'        => esc_html__( 'Block', 'phox-host' ),
                ),
                'default' => 'inline-block',
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'display: {{VALUE}};',
                ),
            )
        );

        $this->add_control(
            'price_prefix_currency_position',
            array(
                'label'   => esc_html__( 'Vertical Currency Position', 'phox-host' ),
                'type'    => Controls_Manager::NUMBER,
                'default' => '-8',
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'top: {{VALUE}}px;',
                ),
            )
        );

        $this->add_control(
            'title_text_image_prefix',
            [
                'label'         => esc_html('Active Text Image', 'phox-host'),
                'type'          => Controls_Manager::SWITCHER,
                'return_value'  =>'yes',
                'default'       =>'no',
            ]
        );

        $this->add_control(
            'title_text_image_choose_styling_prefix',
            [
                'type'          => Controls_Manager::CHOOSE,
                'default'       => 'image',
                'toggle'        => false,
                'options'        =>[
                    'image' =>  [
                        'title' =>  esc_html__('Image', 'phox-host'),
                        'icon'  =>  'eicon-image'
                    ],
                    'gradient' =>  [
                        'title' =>  esc_html__('Gradient', 'phox-host'),
                        'icon'  =>  'eicon-barcode'
                    ],
                ],
                'condition' =>  [
                    'title_text_image_prefix'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_prefix',
            [
                'label'         => esc_html('Color', 'phox-host'),
                'type'          => Controls_Manager::COLOR,
                'default'       => '',
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'gradient',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_stop_prefix',
            [
                'label'         => esc_html('Location', 'phox-host'),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => ['%'],
                'default'       => [
                    'unit'  =>  '%',
                    'size'  =>  0
                ],
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'gradient',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_two_prefix',
            [
                'label'         => esc_html('Second Color', 'phox-host'),
                'type'          => Controls_Manager::COLOR,
                'default'       => '#e3e3e3',
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'gradient',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_two_stop_prefix',
            [
                'label'         => esc_html('Location', 'phox-host'),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => ['%'],
                'default'       => [
                    'unit'  =>  '%',
                    'size'  =>  0
                ],
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'gradient',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_gradient_type_prefix',
            [
                'label'     => esc_html('Type', 'phox-host'),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'linear'    => esc_html('Linear', 'phox-host'),
                    'radial'    => esc_html('Radial', 'phox-host'),
                ],
                'default'       => 'linear',
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'gradient',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_gradient_angle_prefix',
            [
                'label'         => esc_html('Angle', 'phox-host'),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => ['deg'],
                'default'       => [
                    'unit'  =>  'deg',
                    'size'  =>  180
                ],
                'range' => [
                    'step'  =>  'deg'
                ],
                'selectors' =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'background-color: transparent; background-image: linear-gradient({{SIZE}}{{UNIT}}, {{title_text_image_bg_color_prefix.VALUE}} {{title_text_image_bg_color_stop_prefix.SIZE}}{{title_text_image_bg_color_stop_prefix.UNIT}}, {{title_text_image_bg_color_two_prefix.VALUE}} {{title_text_image_bg_color_two_stop_prefix.SIZE}}{{title_text_image_bg_color_two_stop_prefix.UNIT}})',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'gradient',
                    'title_text_image_bg_gradient_type_prefix' => 'linear',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_gradient_position_prefix',
            [
                'label'         => esc_html('Position', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    'center center'    => esc_html('Center Center', 'phox-host'),
                    'center left'      => esc_html('Center Left', 'phox-host'),
                    'center right'     => esc_html('Center Right', 'phox-host'),
                    'top center'       => esc_html('Top Center', 'phox-host'),
                    'top left'         => esc_html('Top Left', 'phox-host'),
                    'top right'        => esc_html('Top Right', 'phox-host'),
                    'bottom center'    => esc_html('Bottom Center', 'phox-host'),
                    'bottom left'      => esc_html('Bottom Left', 'phox-host'),
                    'bottom right'     => esc_html('Bottom Right', 'phox-host')
                ],
                'default'       => 'center center',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'background-color: transparent; background-image: radial-gradient(at {{VALUE}}, {{title_text_image_bg_color_prefix.VALUE}} {{title_text_image_bg_color_stop_prefix.SIZE}}{{title_text_image_bg_color_stop_prefix.UNIT}}, {{title_text_image_bg_color_two_prefix.VALUE}} {{title_text_image_bg_color_two_stop_prefix.SIZE}}{{title_text_image_bg_color_two_stop_prefix.UNIT}})',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'gradient',
                    'title_text_image_bg_gradient_type_prefix' => 'radial',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_prefix',
            [
                'label'     => esc_html('Text Image', 'phox-host'),
                'type'      => Controls_Manager::MEDIA,
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'image',
                    'title_text_image_prefix'  =>  'yes'
                ],
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'background-image:url({{URL}})',
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_position_prefix',
            [
                'label'         => esc_html('Background Position', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    'center center'    => esc_html('Center Center', 'phox-host'),
                    'center left'      => esc_html('Center Left', 'phox-host'),
                    'center right'     => esc_html('Center Right', 'phox-host'),
                    'top center'       => esc_html('Top Center', 'phox-host'),
                    'top left'         => esc_html('Top Left', 'phox-host'),
                    'top right'        => esc_html('Top Right', 'phox-host'),
                    'bottom center'    => esc_html('Bottom Center', 'phox-host'),
                    'bottom left'      => esc_html('Bottom Left', 'phox-host'),
                    'bottom right'     => esc_html('Bottom Right', 'phox-host')
                ],
                'default'       => 'center center',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'background-position: {{VALUE}}',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'image',
                    'title_text_image_prefix'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_repeat_prefix',
            [
                'label'         => esc_html('Background Repeat', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    ''                  => esc_html('Default', 'phox-host'),
                    'no-repeat'         => esc_html('No Repeat', 'phox-host'),
                    'repeat'            => esc_html('Repeat', 'phox-host'),
                    'repeat-x'          => esc_html('Repeat-X', 'phox-host'),
                    'repeat-y'          => esc_html('Repeat-Y', 'phox-host'),
                ],
                'default'       => '',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'background-repeat: {{VALUE}}',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'image',
                    'title_text_image_prefix'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_size_prefix',
            [
                'label'         => esc_html('Background Size', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    ''            => esc_html('Default', 'phox-host'),
                    'auto'        => esc_html('Auto', 'phox-host'),
                    'cover'       => esc_html('Cover', 'phox-host'),
                    'contain'     => esc_html('Contain', 'phox-host'),
                ],
                'default'       => '',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_prefix'] => 'background-size: {{VALUE}}',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_prefix'   =>  'image',
                    'title_text_image_prefix'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'price_discount_style',
            array(
                'label'     => esc_html__( 'Discount', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition' => [
                    'discount' => 'yes',
                ],
            )
        );
        $this->add_control(
            'discount_color',
            [
                'label' => esc_html__( 'Discount Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['price_discount'] => 'color: {{VALUE}}',
                ],
                'condition' => [
                    'discount' => 'yes',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'      => 'price_discount',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector'  => '{{WRAPPER}}  ' . $css_scheme['price_discount'],
                'condition' => [
                    'discount' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'price_val_style',
            array(
                'label'     => esc_html__( 'Price Value', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'price_color',
            array(
                'label' => esc_html__( 'Price Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'default' => '#c51e3a',
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_value'] => 'color: {{VALUE}}',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'      => 'price_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_PRIMARY
                ],
                'selector'  => '{{WRAPPER}}  ' . $css_scheme['price_value'],
            )
        );

        $this->add_control(
            'title_text_image_price',
            [
                'label'         => esc_html('Active Text Image', 'phox-host'),
                'type'          => Controls_Manager::SWITCHER,
                'return_value'  =>'yes',
                'default'       =>'no',
            ]
        );

        $this->add_control(
            'title_text_image_choose_styling_price',
            [
                'type'          => Controls_Manager::CHOOSE,
                'default'       => 'image',
                'toggle'        => false,
                'options'        =>[
                    'image' =>  [
                        'title' =>  esc_html__('Image', 'phox-host'),
                        'icon'  =>  'eicon-image'
                    ],
                    'gradient' =>  [
                        'title' =>  esc_html__('Gradient', 'phox-host'),
                        'icon'  =>  'eicon-barcode'
                    ],
                ],
                'condition' =>  [
                    'title_text_image_price'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_price',
            [
                'label'         => esc_html('Color', 'phox-host'),
                'type'          => Controls_Manager::COLOR,
                'default'       => '',
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'gradient',
                    'title_text_image_price'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_stop_price',
            [
                'label'         => esc_html('Location', 'phox-host'),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => ['%'],
                'default'       => [
                    'unit'  =>  '%',
                    'size'  =>  0
                ],
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'gradient',
                    'title_text_image_price'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_two_price',
            [
                'label'         => esc_html('Second Color', 'phox-host'),
                'type'          => Controls_Manager::COLOR,
                'default'       => '#e3e3e3',
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'gradient',
                    'title_text_image_price'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_color_two_stop_price',
            [
                'label'         => esc_html('Location', 'phox-host'),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => ['%'],
                'default'       => [
                    'unit'  =>  '%',
                    'size'  =>  0
                ],
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'gradient',
                    'title_text_image_price'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_gradient_type_price',
            [
                'label'     => esc_html('Type', 'phox-host'),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'linear'    => esc_html('Linear', 'phox-host'),
                    'radial'    => esc_html('Radial', 'phox-host'),
                ],
                'default'       => 'linear',
                'render_type'   => 'ui',
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'gradient',
                    'title_text_image_price'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_gradient_angle_price',
            [
                'label'         => esc_html('Angle', 'phox-host'),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => ['deg'],
                'default'       => [
                    'unit'  =>  'deg',
                    'size'  =>  180
                ],
                'range' => [
                    'step'  =>  'deg'
                ],
                'selectors' =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_value'] => 'background-color: transparent; background-image: linear-gradient({{SIZE}}{{UNIT}}, {{title_text_image_bg_color_price.VALUE}} {{title_text_image_bg_color_stop_price.SIZE}}{{title_text_image_bg_color_stop_price.UNIT}}, {{title_text_image_bg_color_two_price.VALUE}} {{title_text_image_bg_color_two_stop_price.SIZE}}{{title_text_image_bg_color_two_stop_price.UNIT}})',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'gradient',
                    'title_text_image_bg_gradient_type_price' => 'linear',
                    'title_text_image_price'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_gradient_position_price',
            [
                'label'         => esc_html('Position', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    'center center'    => esc_html('Center Center', 'phox-host'),
                    'center left'      => esc_html('Center Left', 'phox-host'),
                    'center right'     => esc_html('Center Right', 'phox-host'),
                    'top center'       => esc_html('Top Center', 'phox-host'),
                    'top left'         => esc_html('Top Left', 'phox-host'),
                    'top right'        => esc_html('Top Right', 'phox-host'),
                    'bottom center'    => esc_html('Bottom Center', 'phox-host'),
                    'bottom left'      => esc_html('Bottom Left', 'phox-host'),
                    'bottom right'     => esc_html('Bottom Right', 'phox-host')
                ],
                'default'       => 'center center',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_value'] => 'background-color: transparent; background-image: radial-gradient(at {{VALUE}}, {{title_text_image_bg_color_price.VALUE}} {{title_text_image_bg_color_stop_price.SIZE}}{{title_text_image_bg_color_stop_price.UNIT}}, {{title_text_image_bg_color_two_price.VALUE}} {{title_text_image_bg_color_two_stop_price.SIZE}}{{title_text_image_bg_color_two_stop_price.UNIT}})',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'gradient',
                    'title_text_image_bg_gradient_type_price' => 'radial',
                    'title_text_image_price'  =>  'yes'
                ],
                'of_type'   =>  'gradient'
            ]
        );

        $this->add_control(
            'title_text_image_bg_price',
            [
                'label'     => esc_html('Text Image', 'phox-host'),
                'type'      => Controls_Manager::MEDIA,
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'image',
                    'title_text_image_price'  =>  'yes'
                ],
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_value'] => 'background-image:url({{URL}})',
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_position_price',
            [
                'label'         => esc_html('Background Position', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    'center center'    => esc_html('Center Center', 'phox-host'),
                    'center left'      => esc_html('Center Left', 'phox-host'),
                    'center right'     => esc_html('Center Right', 'phox-host'),
                    'top center'       => esc_html('Top Center', 'phox-host'),
                    'top left'         => esc_html('Top Left', 'phox-host'),
                    'top right'        => esc_html('Top Right', 'phox-host'),
                    'bottom center'    => esc_html('Bottom Center', 'phox-host'),
                    'bottom left'      => esc_html('Bottom Left', 'phox-host'),
                    'bottom right'     => esc_html('Bottom Right', 'phox-host')
                ],
                'default'       => 'center center',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_value'] => 'background-position: {{VALUE}}',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'image',
                    'title_text_image_price'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_repeat_price',
            [
                'label'         => esc_html('Background Repeat', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    ''                  => esc_html('Default', 'phox-host'),
                    'no-repeat'         => esc_html('No Repeat', 'phox-host'),
                    'repeat'            => esc_html('Repeat', 'phox-host'),
                    'repeat-x'          => esc_html('Repeat-X', 'phox-host'),
                    'repeat-y'          => esc_html('Repeat-Y', 'phox-host'),
                ],
                'default'       => '',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_value'] => 'background-repeat: {{VALUE}}',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'image',
                    'title_text_image_price'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'title_text_image_bg_size_price',
            [
                'label'         => esc_html('Background Size', 'phox-host'),
                'type'          => Controls_Manager::SELECT,
                'options'   => [
                    ''            => esc_html('Default', 'phox-host'),
                    'auto'        => esc_html('Auto', 'phox-host'),
                    'cover'       => esc_html('Cover', 'phox-host'),
                    'contain'     => esc_html('Contain', 'phox-host'),
                ],
                'default'       => '',
                'selectors'     =>  [
                    '{{WRAPPER}} ' . $css_scheme['price_value'] => 'background-size: {{VALUE}}',
                ],
                'condition'     => [
                    'title_text_image_choose_styling_price'   =>  'image',
                    'title_text_image_price'  =>  'yes'
                ]
            ]
        );

        $this->add_control(
            'price_suffix_style',
            array(
                'label'     => esc_html__( 'Suffix', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'price_suffix_color',
            array(
                'label' => esc_html__( 'Price Suffix Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'default' => '#c51e3a',
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_suffix'] => 'color: {{VALUE}}',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'price_suffix_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['price_suffix'],
            )
        );

        $this->add_control(
            'price_suffix_dispaly',
            array(
                'label'   => esc_html__( 'Suffix Display', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'options' => array(
                    'inline-block' => esc_html__( 'Inline', 'phox-host' ),
                    'block'        => esc_html__( 'Block', 'phox-host' ),
                ),
                'default'   => 'inline-block',
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_suffix'] => 'display: {{VALUE}};',
                ),
            )
        );

        $this->add_control(
            'price_desc_style',
            array(
                'label'     => esc_html__( 'Description', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'price_desc_color',
            array(
                'label' => esc_html__( 'Price Description Color', 'phox-host' ),
                'type'  => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_desc'] => 'color: {{VALUE}}',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'price_desc_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['price_desc'],
            )
        );

        $this->add_control(
            'price_desc_gap',
            array(
                'label' => esc_html__( 'Gap', 'phox-host' ),
                'type'  => Controls_Manager::SLIDER,
                'range' => array(
                    'px' => array(
                        'min' => 0,
                        'max' => 30,
                    ),
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['price_desc'] => 'margin-top: {{SIZE}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'price_padding',
            array(
                'label'      => esc_html__( 'Pricing Block Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['price'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
                'separator' => 'before',
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'           => 'price_border',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'placeholder'    => '1px',
                'selector'       => '{{WRAPPER}} ' . $css_scheme['price'],
            )
        );

        $this->add_responsive_control(
            'price_border_radius',
            array(
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['price'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'price_alignment',
            array(
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => array(
                    'left'    => array(
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-arrow-left',
                    ),
                    'center' => array(
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-justify',
                    ),
                    'right' => array(
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-arrow-right',
                    ),
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['price'] => 'text-align: {{VALUE}};',
                ),
            )
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_features_style',
            array(
                'label'      => esc_html__( 'Features', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            )
        );

        $this->add_control(
            'features_header_style',
            array(
                'label'     => esc_html__( 'Features Header', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'features_header_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['features_header'],
            )
        );

        $this->add_control(
            'features_header_color',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['features_header'] => 'color: {{VALUE}}',
                ),
            ]
        );

        $this->add_responsive_control(
            'features_header_alignment',
            array(
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => array(
                    'left'    => array(
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-arrow-left',
                    ),
                    'center' => array(
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-justify',
                    ),
                    'right' => array(
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-arrow-right',
                    ),
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['features_header'] => 'text-align: {{VALUE}};',
                ),
            )
        );

        $this->add_responsive_control(
            'features_header_padding',
            array(
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['features_header'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_control(
            'feature_item_style',
            [
                'label'     => esc_html__( 'Feature Item', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'features_bg_color',
            array(
                'label' => esc_html__( 'Background Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['features_wrap'] => 'background-color: {{VALUE}}',
                ),
            )
        );

        $this->add_responsive_control(
            'features_padding',
            array(
                'label'      => esc_html__( 'Features Block Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['features_wrap'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'           => 'features_border',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'placeholder'    => '1px',
                'selector'       => '{{WRAPPER}} ' . $css_scheme['features_wrap'],
            )
        );

        $this->add_responsive_control(
            'features_border_radius',
            array(
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['features_wrap'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'features_alignment',
            array(
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => array(
                    'left'    => array(
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-arrow-left',
                    ),
                    'center' => array(
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-justify',
                    ),
                    'right' => array(
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-arrow-right',
                    ),
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['features_wrap'] => 'text-align: {{VALUE}};',
                ),
            )
        );

        $this->start_controls_tabs( 'feature_item_effects' );
        $this->start_controls_tab(
            'feature_icon_normal',
            [
                'label' => esc_html__( 'Normal', 'phox-host' ),
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'features_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['features_item'],
            )
        );

        $this->add_control(
            'feature_color',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_SECONDARY
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['feature_item'] => 'color: {{VALUE}}',
                ),
                'default' => '#708791',
            ]
        );

        $this->add_control(
            'feature_icon_size_text_indent',
            [
                'label'   => esc_html__( 'Text Indent', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'range' => array(
                    'px' => array(
                        'min' => 0,
                        'max' => 90,
                    ),
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['feature_item_text']  => 'padding-left: {{SIZE}}{{UNIT}};',
                ),
            ]
        );
        $this->end_controls_tab();

        $this->start_controls_tab(
            'feature_icon_hover',
            [
                'label' => esc_html__( 'Hover', 'phox-host' ),
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'features_typography_hover',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['features_item'] . ':hover',
            )
        );

        $this->add_control(
            'feature_color_hover',
            [
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_SECONDARY
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['feature_item'] . ':hover' => 'color: {{VALUE}}',
                ),
                'default' => '#708791',
            ]
        );

        $this->add_control(
            'feature_icon_size_text_indent_hover',
            [
                'label'   => esc_html__( 'Text Indent', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'range' => array(
                    'px' => array(
                        'min' => 0,
                        'max' => 90,
                    ),
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['feature_item_text'] . ':hover'  => 'padding-left: {{SIZE}}{{UNIT}};',
                ),
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->add_control(
            'feature_icon_style',
            [
                'label'     => esc_html__( 'Feature Icon', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'feature_icon_size',
            [
                'label'   => esc_html__( 'Icon Size', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'default' => array(
                    'size' => 14,
                    'unit' => 'px',
                ),
                'range' => array(
                    'px' => array(
                        'min' => 6,
                        'max' => 90,
                    ),
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['feature_item'] . ' .feature-icon:before' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} ' . $css_scheme['feature_item'] . ' .feature-icon' => 'width: {{SIZE}}{{UNIT}};',
                ),
            ]
        );

        $this->add_control(
            'feature_icon_color',
            [
                'label' => esc_html__( 'Icon Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_ACCENT
                ],
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['feature_item'] . ' .feature-icon:before' => 'color: {{VALUE}}',
                ),
                'default' => '#c51e3a',
            ]
        );

        $this->add_responsive_control(
            'feature_icon_margin',
            [
                'label'      => __( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'default'    => [
                    'top' => 0,
                    'left'  => is_rtl() ? 5 : 0 ,
                    'right' => is_rtl() ? 0 : 5 ,
                    'bottom' => 0,
                    'unit' => 'px'
                ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item'] . ' .feature-icon' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'feature_image_style',
            array(
                'label'     => esc_html__( 'Feature Image', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'feature_image_size',
            array(
                'label'   => esc_html__( 'Image Size', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'default' => array(
                    'size' => 14,
                    'unit' => 'px',
                ),
                'range' => array(
                    'px' => array(
                        'min' => 0,
                        'max' => 90,
                    ),
                    '%' => array(
                        'min' => 0,
                        'max' => 100,
                    ),
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['feature_item'] . ' .feature-image' => 'width: {{SIZE}}{{UNIT}};',
                ),
            )
        );

        $this->add_control(
            'features_description_style',
            [
                'label'     => esc_html__( 'Feature Description', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_responsive_control(
            'features_description_style_icon_size',
            [
                'label'      => esc_html__( 'Button Size', 'phox-host' ),
                'type'       => Controls_Manager::SLIDER,
                'size_units' => ['px'],
                'range'      => [
                    'px' => [
                        'min' => 10,
                        'max' => 100,
                    ],
                ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_button'] => 'font-size: {{SIZE}}{{UNIT}};',
                ]
            ]
        );

        $this->add_control(
            'features_description_style_icon_color',
            [
                'label'      => esc_html__( 'Button Color', 'phox-host' ),
                'type'       => Controls_Manager::COLOR,
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_button'] => 'color: {{VALUE}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'features_description_style_icon_margin',
            [
                'label'      => __( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'default'    => [
                    'top' => 0,
                    'left'  => is_rtl() ? 0 : 5 ,
                    'right' => is_rtl() ? 5 : 0 ,
                    'bottom' => 0,
                    'unit' => 'px'
                ],
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_button'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'features_description_style_content_background_color',
            [
                'label'      => esc_html__( 'Content Background Color', 'phox-host' ),
                'type'       => Controls_Manager::COLOR,
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_content'] => 'background-color: {{VALUE}};',
                ]
            ]
        );

        $this->add_control(
            'features_description_style_content_color',
            [
                'label'      => esc_html__( 'Content Color', 'phox-host' ),
                'type'       => Controls_Manager::COLOR,
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_content'] => 'color: {{VALUE}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'features_description_style_content_width',
            [
                'label'   => esc_html__( 'Content Width', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 500,
                        'step' => 5,
                    ],
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_content'] => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->add_control(
            'features_description_style_arrow_color',
            [
                'label'      => esc_html__( 'Arrow Color', 'phox-host' ),
                'type'       => Controls_Manager::COLOR,
                'selectors'  => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_content'] . '::after' => 'border-color: transparent transparent {{VALUE}} transparent;',
                ]
            ]
        );

        $this->add_responsive_control(
            'features_description_style_arrow_position',
            [
                'label'   => esc_html__( 'Arrow Position', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range'   => [
                    'px' => [
                        'min'  => 0,
                        'max'  => 500,
                        'step' => 5,
                    ],
                    '%' => [
                        'min'  => 0,
                        'max'  => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['feature_item_description_content'] . '::after' => 'margin-left: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'      => 'features_description_style_content_typo',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector'  => '{{WRAPPER}} ' . $css_scheme['feature_item_description_content'],
            ]
        );

        $this->add_responsive_control(
            'features_description_style_content_padding',
            [
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors'  => [
                    '{{WRAPPER}} '  . $css_scheme['feature_item_description_content'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ]
            ]
        );

        $this->add_responsive_control(
            'features_description_style_content_direction',
            [
                'label'     => __('Direction', 'phox-host'),
                'type'      => Controls_Manager::CHOOSE,
                'default'   => 'down',
                'options'   => [
                    'left'    => [
                        'title' =>  __('Left', 'phox-host'),
                        'icon'  =>  'eicon-arrow-left'
                    ],
                    'top'    => [
                        'title' =>  __('Top', 'phox-host'),
                        'icon'  =>  'eicon-arrow-up'
                    ],
                    'right'    => [
                        'title' =>  __('Right', 'phox-host'),
                        'icon'  =>  'eicon-arrow-right'
                    ],
                    'down'    => [
                        'title' =>  __('Down', 'phox-host'),
                        'icon'  =>  'eicon-arrow-down'
                    ],
                ]
            ]
        );

        $this->add_control(
            'features_divider_style',
            [
                'label'     => esc_html__( 'Features Divider', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'features_divider',
            [
                'label'        => esc_html__( 'Divider', 'phox-host' ),
                'type'         => Controls_Manager::SWITCHER,
                'label_on'     => esc_html__( 'Yes', 'phox-host' ),
                'label_off'    => esc_html__( 'No', 'phox-host' ),
                'return_value' => 'yes',
                'default'      => '',
            ]
        );

        $this->add_control(
            'features_divider_line',
            [
                'label' => esc_html__( 'Style', 'phox-host' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'solid' => esc_html__( 'Solid', 'phox-host' ),
                    'double' => esc_html__( 'Double', 'phox-host' ),
                    'dotted' => esc_html__( 'Dotted', 'phox-host' ),
                    'dashed' => esc_html__( 'Dashed', 'phox-host' ),
                ],
                'default' => 'solid',
                'condition' => [
                    'features_divider' => 'yes',
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['features_item'] . ':before' => 'border-top-style: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'features_divider_color',
            array(
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT
                ],
                'condition' => array(
                    'features_divider' => 'yes',
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['features_item'] . ':before' => 'border-top-color: {{VALUE}};',
                ),
            )
        );

        $this->add_control(
            'features_divider_weight',
            array(
                'label'   => esc_html__( 'Weight', 'phox-host' ),
                'type'    => Controls_Manager::SLIDER,
                'default' => array(
                    'size' => 1,
                    'unit' => 'px',
                ),
                'range' => array(
                    'px' => array(
                        'min' => 1,
                        'max' => 10,
                    ),
                ),
                'condition' => array(
                    'features_divider' => 'yes',
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['features_item'] . ':before' => 'border-top-width: {{SIZE}}{{UNIT}};',
                ),
            )
        );

        $this->add_control(
            'features_divider_width',
            array(
                'label' => esc_html__( 'Width', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'condition' => array(
                    'features_divider' => 'yes',
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['features_item'] . ':before' => 'width: {{SIZE}}%',
                ),
            )
        );

        $this->add_control(
            'features_divider_gap',
            array(
                'label' => esc_html__( 'Gap', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'default' => array(
                    'size' => 15,
                    'unit' => 'px',
                ),
                'range' => array(
                    'px' => array(
                        'min' => 1,
                        'max' => 50,
                    ),
                ),
                'condition' => array(
                    'features_divider' => 'yes',
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['features_item'] . ':before' => 'margin-top: {{SIZE}}{{UNIT}}; margin-bottom: {{SIZE}}{{UNIT}}',
                ),
            )
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_footer_style',
            [
                'label'      => esc_html__( 'Footer', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
            ]
        );

        $this->add_control(
            'action_bg_color',
            array(
                'label' => esc_html__( 'Background Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['footer'] => 'background-color: {{VALUE}}',
                ),
            )
        );

        $this->add_control(
            'action_color',
            array(
                'label' => esc_html__( 'Color', 'phox-host' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['footer'] => 'color: {{VALUE}}',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'action_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['footer'],
            )
        );

        $this->add_responsive_control(
            'action_padding',
            array(
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['footer'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'action_margin',
            array(
                'label'      => esc_html__( 'Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['footer'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'           => 'action_border',
                'label'          => esc_html__( 'Border', 'phox-host' ),
                'placeholder'    => '1px',
                'selector'       => '{{WRAPPER}} ' . $css_scheme['footer'],
            )
        );

        $this->add_responsive_control(
            'action_border_radius',
            array(
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['footer'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'action_alignment',
            array(
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => array(
                    'left'    => array(
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-arrow-left',
                    ),
                    'center' => array(
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-justify',
                    ),
                    'right' => array(
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-arrow-right',
                    ),
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['footer'] => 'text-align: {{VALUE}};',
                ),
            )
        );

        $this->add_control(
            'footer_button',
            array(
                'label'     => esc_html__( 'Button', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'separator' => 'before',
            )
        );

        $this->add_control(
            'button_size',
            array(
                'label'   => esc_html__( 'Size', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'auto',
                'options' => array(
                    'auto' => esc_html__( 'auto', 'phox-host' ),
                    'full'  => esc_html__( 'full', 'phox-host' ),
                ),
            )
        );

        $this->add_control(
            'add_button_icon',
            array(
                'label'        => esc_html__( 'Add Icon', 'phox-host' ),
                'type'         => Controls_Manager::SWITCHER,
                'label_on'     => esc_html__( 'Yes', 'phox-host' ),
                'label_off'    => esc_html__( 'No', 'phox-host' ),
                'return_value' => 'yes',
                'default'      => '',
            )
        );

        $this->add_control(
            'button_icon',
            array(
                'label'       => esc_html__( 'Icon', 'phox-host' ),
                'type'        => Controls_Manager::ICONS,
                'label_block' => true,
                'file'        => '',
                'default'     => [
                    'value' =>'fas fa-shopping-cart'
                ],
                'condition' => array(
                    'add_button_icon' => 'yes',
                ),
            )
        );

        $this->add_control(
            'button_icon_position',
            array(
                'label'   => esc_html__( 'Icon Position', 'phox-host' ),
                'type'    => Controls_Manager::SELECT,
                'options' => array(
                    'left'  => esc_html__( 'Before Text', 'phox-host' ),
                    'right' => esc_html__( 'After Text', 'phox-host' ),
                ),
                'default'     => 'left',
                'render_type' => 'template',
                'selectors'   => array(
                    '{{WRAPPER}} ' . $css_scheme['button_icon'] => 'float: {{VALUE}}',
                ),
                'condition' => array(
                    'add_button_icon' => 'yes',
                ),
            )
        );

        $this->add_control(
            'button_icon_size',
            array(
                'label' => esc_html__( 'Icon Size', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => array(
                    'px' => array(
                        'min' => 7,
                        'max' => 90,
                    ),
                ),
                'condition' => array(
                    'add_button_icon' => 'yes',
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['button_icon'] => 'font-size: {{SIZE}}{{UNIT}};',
                ),
            )
        );

        $this->add_control(
            'button_icon_color',
            array(
                'label'     => esc_html__( 'Icon Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'condition' => array(
                    'add_button_icon' => 'yes',
                ),
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['button_icon'] => 'color: {{VALUE}}',
                ),
            )
        );

        $this->add_responsive_control(
            'button_icon_margin',
            array(
                'label'      => esc_html__( 'Icon Margin', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'condition' => array(
                    'add_button_icon' => 'yes',
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['button_icon'] => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->start_controls_tabs( 'tabs_button_style' );

        $this->start_controls_tab(
            'tab_button_normal',
            array(
                'label' => esc_html__( 'Normal', 'phox-host' ),
            )
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            array(
                'name'     => 'button_bg',
                'selector' => '{{WRAPPER}} ' . $css_scheme['button'],
                'types'    => [ 'classic', 'gradient' ],
            )
        );

        $this->add_control(
            'button_color',
            array(
                'label'     => esc_html__( 'Text Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'color: {{VALUE}}',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'button_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_ACCENT
                ],
                'selector' => '{{WRAPPER}}  ' . $css_scheme['button'],
            )
        );

        $this->add_responsive_control(
            'button_padding',
            array(
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'button_border_radius',
            array(
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['button'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'        => 'button_border',
                'label'       => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default'     => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['button'],
            )
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            array(
                'name'     => 'button_box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['button'],
            )
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'tab_button_hover',
            array(
                'label' => esc_html__( 'Hover', 'phox-host' ),
            )
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            array(
                'name'     => 'button_hover_bg',
                'selector' => '{{WRAPPER}} ' . $css_scheme['button'] . ':hover',
                'fields_options' => array(
                    'background' => array(
                        'default' => 'classic',
                    ),
                    'color' => array(
                        'label' => _x( 'Background Color', 'Background Control', 'phox-host' ),
                    ),
                    'color_b' => array(
                        'label' => _x( 'Second Background Color', 'Background Control', 'phox-host' ),
                    ),
                ),
                'exclude' => array(
                    'image',
                    'position',
                    'attachment',
                    'attachment_alert',
                    'repeat',
                    'size',
                ),
            )
        );

        $this->add_control(
            'button_hover_color',
            array(
                'label'     => esc_html__( 'Text Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['button'] . ':hover' => 'color: {{VALUE}}',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            array(
                'name'     => 'button_hover_typography',
                'selector' => '{{WRAPPER}}  ' . $css_scheme['button'] . ':hover',
            )
        );

        $this->add_responsive_control(
            'button_hover_padding',
            array(
                'label'      => esc_html__( 'Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['button'] . ':hover' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'button_hover_border_radius',
            array(
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['button'] . ':hover' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'        => 'button_hover_border',
                'label'       => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default'     => '1px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['button'] . ':hover',
            )
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            array(
                'name'     => 'button_hover_box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['button'] . ':hover',
            )
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

        $this->start_controls_section(
            'section_strip_style',
            array(
                'label'      => esc_html__( 'Strip', 'phox-host' ),
                'tab'        => Controls_Manager::TAB_STYLE,
                'show_label' => false,
                'condition' => array(
                    'strip' => 'yes',
                ),
            )
        );

        $this->add_control(
            'strip_default_style',
            [
                'label'     => esc_html__( 'Default', 'phox-host' ),
                'type'      => Controls_Manager::HEADING,
                'condition' => array(
                    'price_strip_type' => 'strip_default',
                ),
            ]
        );

        $this->add_control(
            'strip_default_bg',
            [
                'label'     => esc_html__( 'Background Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['strip_default'] => 'background-color: {{VALUE}}',
                ),
                'condition' => array(
                    'price_strip_type' => 'strip_default',
                ),
            ]
        );

        $this->add_control(
            'strip_default_text_color',
            [
                'label'     => esc_html__( 'Text Color', 'phox-host' ),
                'type'      => Controls_Manager::COLOR,
                'default'   => '#c51e3a',
                'selectors' => array(
                    '{{WRAPPER}} ' . $css_scheme['strip_default'] => 'color: {{VALUE}}',
                ),
                'condition' => array(
                    'price_strip_type' => 'strip_default',
                ),
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'  => 'strip_typography',
                'selector' => '{{WRAPPER}} ' . $css_scheme['strip_default'],
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_ACCENT
                ],
                'condition' => array(
                    'price_strip_type' => 'strip_default',
                ),
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name'  => 'box_shadow',
                'selector' => '{{WRAPPER}} ' . $css_scheme['strip_default'],
                'condition' => array(
                    'price_strip_type' => 'strip_default',
                ),
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            array(
                'name'        => 'strip_box_border',
                'label'       => esc_html__( 'Border', 'phox-host' ),
                'placeholder' => '1px',
                'default'     => '2px',
                'selector'    => '{{WRAPPER}} ' . $css_scheme['strip_default'],
            )
        );

        $this->add_responsive_control(
            'strip_border_radius',
            array(
                'label'      => esc_html__( 'Border Radius', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['strip_default'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'strip_padding',
            array(
                'label'      => esc_html__( 'Strip Padding', 'phox-host' ),
                'type'       => Controls_Manager::DIMENSIONS,
                'size_units' => array( 'px', '%', 'em' ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['strip_default'] => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ),
            )
        );

        $this->add_responsive_control(
            'strip_alignment',
            array(
                'label'   => esc_html__( 'Alignment', 'phox-host' ),
                'type'    => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => array(
                    'left'    => array(
                        'title' => esc_html__( 'Left', 'phox-host' ),
                        'icon'  => 'eicon-arrow-left',
                    ),
                    'center' => array(
                        'title' => esc_html__( 'Center', 'phox-host' ),
                        'icon'  => 'eicon-text-align-justify',
                    ),
                    'right' => array(
                        'title' => esc_html__( 'Right', 'phox-host' ),
                        'icon'  => 'eicon-arrow-right',
                    ),
                ),
                'selectors'  => array(
                    '{{WRAPPER}} ' . $css_scheme['strip_default_main'] => 'text-align: {{VALUE}};',
                ),
            )
        );

        $this->add_responsive_control(
            'strip_rotate',
            [
                'label' => esc_html__( 'Rotate', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 360,
                        'step' => 5,
                    ],
                ],
                'default' => [
                    'size' => '0',
                ],
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['strip_default_main'] => 'transform: rotate({{SIZE}}deg)',
                ],

            ]
        );

        $this->add_responsive_control(
            'strip_element_width',
            [
                'label' => esc_html__( 'Width', 'phox-host' ),
                'type' => Controls_Manager::SELECT,
                'default' => '',
                'options' => [
                    '' => esc_html__( 'Default', 'phox-host' ),
                    'inherit' => esc_html__( 'Full Width', 'phox-host' ) . ' (100%)',
                    'auto' => esc_html__( 'Inline', 'phox-host' ) . ' (auto)',
                    'initial' => esc_html__( 'Custom', 'phox-host' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['strip_default_main'] => 'width: {{VALUE}}; max-width: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'strip_element_custom_width',
            [
                'label' => esc_html__( 'Custom Width', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'condition' => [
                    'strip_element_width' => 'initial',
                ],
                'size_units' => [ 'px', '%', 'vw' ],
                'selectors' => [
                    '{{WRAPPER}} ' . $css_scheme['strip_default_main'] => 'width: {{SIZE}}{{UNIT}}; max-width: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        $start = is_rtl() ? esc_html__( 'Right', 'phox-host' ) : esc_html__( 'Left', 'phox-host' );
        $end = ! is_rtl() ? esc_html__( 'Right', 'phox-host' ) : esc_html__( 'Left', 'phox-host' );

        $this->add_control(
            'strip_offset_orientation_h',
            [
                'label' => esc_html__( 'Horizontal Orientation', 'phox-host' ),
                'type' => Controls_Manager::CHOOSE,
                'toggle' => false,
                'default' => 'start',
                'options' => [
                    'start' => [
                        'title' => $start,
                        'icon' => 'eicon-h-align-left',
                    ],
                    'end' => [
                        'title' => $end,
                        'icon' => 'eicon-h-align-right',
                    ],
                ],
                'classes' => 'elementor-control-start-end',
                'render_type' => 'ui'
            ]
        );

        $this->add_responsive_control(
            'strip_offset_x',
            [
                'label' => esc_html__( 'Offset', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vw' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vh' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                ],
                'default' => [
                    'size' => '10',
                ],
                'size_units' => [ 'px', '%', 'vw', 'vh' ],
                'selectors' => [
                    'body:not(.rtl) {{WRAPPER}} '. $css_scheme['strip_default_main'] => 'left: {{SIZE}}{{UNIT}}',
                    'body.rtl {{WRAPPER}} '. $css_scheme['strip_default_main'] => 'right: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'strip_offset_orientation_h!' => 'end'
                ],
            ]
        );

        $this->add_responsive_control(
            'strip_offset_x_end',
            [
                'label' => esc_html__( 'Offset', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 0.1,
                    ],
                    '%' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vw' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vh' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                ],
                'default' => [
                    'size' => '10',
                ],
                'size_units' => [ 'px', '%', 'vw', 'vh' ],
                'selectors' => [
                    'body:not(.rtl) {{WRAPPER}} '. $css_scheme['strip_default_main'] => 'right: {{SIZE}}{{UNIT}}',
                    'body.rtl {{WRAPPER}} '. $css_scheme['strip_default_main'] => 'left: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'strip_offset_orientation_h' => 'end',
                ],
            ]
        );

        $this->add_control(
            'strip_offset_orientation_v',
            [
                'label' => esc_html__( 'Vertical Orientation', 'phox-host' ),
                'type' => Controls_Manager::CHOOSE,
                'toggle' => false,
                'default' => 'start',
                'options' => [
                    'start' => [
                        'title' => esc_html__( 'Top', 'phox-host' ),
                        'icon' => 'eicon-v-align-top',
                    ],
                    'end' => [
                        'title' => esc_html__( 'Bottom', 'phox-host' ),
                        'icon' => 'eicon-v-align-bottom',
                    ],
                ],
                'render_type' => 'ui',
            ]
        );

        $this->add_responsive_control(
            'strip_offset_y',
            [
                'label' => esc_html__( 'Offset', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vh' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vw' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                ],
                'size_units' => [ 'px', '%', 'vh', 'vw' ],
                'default' => [
                    'size' => '10',
                ],
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['strip_default_main'] => 'top: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'strip_offset_orientation_v!' => 'end',
                ],
            ]
        );

        $this->add_responsive_control(
            'strip_offset_y_end',
            [
                'label' => esc_html__( 'Offset', 'phox-host' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vh' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                    'vw' => [
                        'min' => -200,
                        'max' => 200,
                    ],
                ],
                'size_units' => [ 'px', '%', 'vh', 'vw' ],
                'default' => [
                    'size' => '10',
                ],
                'selectors' => [
                    '{{WRAPPER}} '. $css_scheme['strip_default_main'] => 'bottom: {{SIZE}}{{UNIT}}',
                ],
                'condition' => [
                    'strip_offset_orientation_v' => 'end',
                ],
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Render section header widget output on the frontend.
     *
     * Written in PHP and used to generate the final HTML.
     *
     * @since 1.0.0
     * @access protected
     */
    protected function render() {
        $settings = $this->get_settings_for_display();
        $product_id = $settings['whmcs_settings_select_product'];
        ?>

        <div class="wdes-pricing-table">
            <div class = "wdes-pricing-table-heading">
                <?php
				if ( $settings['above_title'] === 'image' ) {

					$this->render_html_block( $settings['image']['url'], '<img class="wdes-pricing-table-img" src="%s" />' );

                } else {

                if( ! empty( $settings['icon']['value'] ) ){
                $this->render_html_block( $settings['icon']['value'], '<div class="wdes-pricing-table-icon" ><div class="wdes-pricing-table-icon-box" ><i class="%s" ></i></div></div>' );
                }

                }
                $base_title = $settings['title_'.$product_id] ?? 'Product Name';
                ?>
                <?php printf('<%2$s class="wdes-pricing-table-title" >%1$s</%2$s>',$base_title, $settings['title_html_tag'] ); ?>
                <?php $this->render_html_block( $settings['subtitle'], '<h4 class="wdes-pricing-table-subtitle" >%s</h4>' ); ?>
            </div>

            <div class="wdes-pricing-table-price" >
                <?php
                $base_perfix = $settings['perfix_'.$settings['currency_price']] ?? '$';
                if(Helper::check_var_true( $settings['title_text_image_prefix'])){
	                $this->add_render_attribute(
                    'price_perfix',
                    'class',
                    [ 'wdes-pricing-table-price-perfix wdes-section-header-primary-label']
                    );
                    }else{ $this->add_render_attribute(
                    'price_perfix',
                    'class',
                    [ 'wdes-pricing-table-price-perfix']) ;
                }
                ?>
                <?php echo sprintf('<span %2$s >%1$s</span>', $base_perfix, $this->get_render_attribute_string( 'price_perfix' )) ?>
                <?php

                $base_price = $this->get_price($product_id, $settings['currency_price'], $settings['payment_type']);
                if(Helper::check_var_true( $settings['title_text_image_price'])){
                    $this->add_render_attribute(
                        'price_price',
                        'class',
                        [ 'wdes-pricing-table-price-num wdes-section-header-primary-label']
                    );
                    }else{
                    $this->add_render_attribute(
                        'price_price',
                        'class',
                        [ 'wdes-pricing-table-price-num']
                    );
                }
                ?>
                <?php echo sprintf( '<span %2$s >%1$s</span>',$base_price, $this->get_render_attribute_string( 'price_price' )); ?>
                <?php
				if ( helpers::check_var_true( $settings['discount'] ) ) {
					$this->render_html_block( $settings['original_price'], '<span class="wdes-pricing-table-price-original" >%s</span>' );
                }
                ?>
                <?php $this->render_html_block( $settings['suffix'], '<span class="wdes-pricing-table-price-suffix" >%s</span>' ); ?>
                <?php $this->render_html_block( $settings['desc'], '<p class="wdes-pricing-table-price-desc" >%s</p>' ); ?>
            </div>

            <div class="wdes-pricing-table-features">
                <?php $this->render_html_block( $settings['feature_header'], '<span class="wdes-pricing-table-features-header" >%s</span>' ); ?>
                <div class="wdes-pricing-table-features-wrap">
                    <?php
                    $items = $settings['feature_items_'.$product_id] ?? '';
                    if ( is_array( $items ) && ! empty($items) ) {

                        foreach ( $items as $item ) {
                            $class = 'pricing-feature-' . $item['_id']
                            ?>

                            <div class="wdes-pricing-feature <?php echo $class; ?>">
                                <div class="wdes-pricing-feature-inner" >
                                    <?php
									if ( $item['item_decoration'] === 'image' ) {
										printf( '<img src="%s" class="feature-image" >', $item['item_image']['url'] );
                                    } else {
                                    printf( '<i class="feature-icon %s" ></i>', $item['item_icon']['value'] );
                                    }

                                    $this->render_html_block( $item['item_title'], '<span class="wdes-pricing-table-features-text" >%s</span>' );

                                    if( ! empty($item['item_description'] ) ){
                                    printf('<div class="extra-info %s">', $settings['features_description_style_content_direction']);
                                        printf('<button class="extra-info-trigger"><i class="%s %s"></i></button>', $item['item_description_icon']['library'], $item['item_description_icon']['value'] );
                                        printf('<span class="extra-info-content">%s</span>', $item['item_description']);
                                        print('</div>');
                                    }
                                    ?>
                                </div>
                            </div>

                            <?php
                        }
                    }
                    ?>
                </div>
            </div>
            <div class="wdes-pricing-feature-footer">
                <?php $this->render_html_block( $settings['button_before'], '<span class="wdes-pricing-table-footer-before" >%s</span>' ); ?>
                <?php
                $base_button_link = $settings['button_link_'.$product_id] ?? [];
				if ( ! empty( $base_button_link ) ) {

					$this->add_render_attribute(
                'footer-button', [
                'class' => [
                'pricing-table-button',
                'button-' . $settings['button_size'] . '-size',
                ],
                'href' => $base_button_link['url'],
                ]
                );

                if ( $base_button_link['nofollow'] ) {
                $this->add_render_attribute( 'footer-button', 'rel', 'nofollow' );
                }

                if ( ! empty( $base_button_link['is_external'] ) ) {
                $this->add_render_attribute( 'footer-button', 'target', '_blank' );
                }

                $button_text = $settings['button_text'];

                if ( helpers::check_var_true( $settings['add_button_icon'] ) ) {

                switch ( $settings['button_icon_position'] ) {
                case 'left':
                $button_text = '<i class="pricing-table-button-icon ' . $settings['button_icon'] . '" ></i>' . $settings['button_text'];
                break;

                case 'right':
                $button_text = $settings['button_text'] . ' <i class="pricing-table-button-icon ' . $settings['button_icon'] . '" ></i> ';
                break;
                }
                }

                printf( '<a %1$s >%2$s</a>',
                $this->get_render_attribute_string( 'footer-button' ),
                $button_text
                );
                }
                ?>
                <?php $this->render_html_block( $settings['button_after'], '<span class="wdes-pricing-table-footer-after" >%s</span>' ); ?>

            </div>
            <?php
            if ( helpers::check_var_true( $settings['strip'] ) ) {

                switch ( $settings['price_strip_type'] ) {

                    case 'strip_default':
                        if ( ! empty( $settings['strip_default_title'] ) ) {

                            $this->add_render_attribute(
                                'strip_default',
                                'class',
                                [ 'wdes-pricing-table-strip-default']
                            );

                            $this->add_render_attribute(
                                'strip_default_title',
                                'class',
                                'wdes-pricing-table-strip-default-inner'
                            );

                            echo sprintf('<div %1$s><div %2$s>%3$s</div></div>',
                                $this->get_render_attribute_string( 'strip_default' ),
                                $this->get_render_attribute_string( 'strip_default_title' ),
                                $settings['strip_default_title']
                            );
                        }

                        break;

                    case 'strip_custom':
                        $this->add_render_attribute(
                            'strip_default',
                            'class',
                            [ 'wdes-pricing-table-strip-default']
                        );

                        $this->add_render_attribute(
                            'strip_custom_image',
                            'class',
                            [ 'wdes-pricing-table-strip-custom-image' ]
                        );

                        echo sprintf('<div %3$s><img src="%2$s" %1$s ></div>',
                            $this->get_render_attribute_string( 'strip_custom_image' ),
                            $settings['strip_custom_image']['url'],
                            $this->get_render_attribute_string( 'strip_default' )
                        );
                        break;
                }
            }
            ?>
        </div>
        <?php

    }




}
